/*
	flounder - copyright 2002 Edward J. Vigmond

	This file is part of flounder.     
	
	flounder is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.     
	
	flounder is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.     
	You should have received a copy of the GNU General Public License
    along with flounder; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "Colourscale.h"

Colourscale :: Colourscale( int ts, CScale_t tcs )
{
	n = 0;
	mindat = 0;
	maxdat = 1;
	scaletype = tcs;
	size( ts );
}


void Colourscale :: calibrate( double tmin, double tmax )
{
	mindat = tmin;
	maxdat = tmax;

	if( maxdat != mindat ) {
		a = static_cast<float>(n)/(maxdat-mindat);
		b = static_cast<float>(n) - a*maxdat;
	}else{
		a = 0.;
		b = 1.;
	}		
}

void Colourscale :: scale( CScale_t cs )
{
	int i;
	float intrvl;

	float ispan = float(n);
	scaletype = cs;	
	switch( cs ) {
		case CS_GREY: 							/* bw */
			for( i=0; i<n; i++ )
				cmap[i][0] = cmap[i][1] = cmap[i][2] = ((float)i)/ispan;
			break;
		case CS_HOT:
			intrvl = ispan/3.;
			for( i=0; i<intrvl; i++ ){
				cmap[i][0] = (float)(i)/(intrvl-1);
				cmap[i][1] = cmap[i][2] = 0; 
			}
			for( ; i<2*intrvl; i++ ) {
				cmap[i][0] = 1;
				cmap[i][1] = (float)(i-intrvl)/(intrvl-1);
				cmap[i][2] = 0; 
			}
			for( ; i<n; i++ ) {
				cmap[i][0] = 1;
				cmap[i][1] = 1; 
				cmap[i][2] = (float)(i-2*intrvl)/(intrvl-1);
			}
			break;
		case CS_RAINBOW:
			intrvl = ispan/6.;
			for( i=0; i<intrvl; i++ ){
				cmap[i][0] =  1-(float)(i)/intrvl;
				cmap[i][1] = 0; 
				cmap[i][2] = 1l;
			}
			for( ; i<2*intrvl; i++ ){
				cmap[i][0] = 0;
				cmap[i][1] = (float)(i-intrvl)/intrvl; 
				cmap[i][2] = 1;
			}
			for( ; i<3*intrvl; i++ ){
				cmap[i][0] = 0;
				cmap[i][1] = 1; 
				cmap[i][2] = 1-(float)(i-2*intrvl)/intrvl;
			}
			for( ; i<4*intrvl; i++ ){
				cmap[i][0] = (float)(i-3*intrvl)/intrvl; 
				cmap[i][1] = 1; 
				cmap[i][2] = 0;
			}
			for( ; i<5*intrvl; i++ ){
				cmap[i][0] = 1; 
				cmap[i][1] = 1-(float)(i-4*intrvl)/intrvl; 
				cmap[i][2] = 0;
			}
			for( ; i<n; i++ ){
				cmap[i][0] = 1; 
				cmap[i][1] = 0; 
				cmap[i][2] =  (float)(i-5*intrvl)/intrvl;
			}
			break;
		case CS_COLD_HOT:
			//intrvl = ispan/2.;
			intrvl = n/2;
			for( i=0; i<intrvl; i++ ){
				cmap[i][2] =  1;
				cmap[i][0] =  cmap[i][1] = (float)(i)/intrvl ;
			}
			for( ; i<n; i++ ){
				cmap[i][0] = 1; 
				cmap[i][1] =  cmap[i][2] = (float)(n-1-i)/intrvl ;
			}
			break;
		case CS_BL_RAINBOW:
		default:
			intrvl = ispan/6.;
			for( i=0; i<intrvl; i++ ){
				cmap[i][0] = 0; 
				cmap[i][1] = 0; 
				cmap[i][2] = (float)(i)/intrvl ;
			}
			for( ; i<2*intrvl; i++ ){
				cmap[i][0] = 0; 
				cmap[i][1] = (float)(i-intrvl)/intrvl; 
				cmap[i][2] = 1;
			}
			for( ; i<3*intrvl; i++ ){
				cmap[i][0] = 0; 
				cmap[i][1] = 1; 
				cmap[i][2] = (float)(3*intrvl-i)/intrvl;
			}
			for( ; i<4*intrvl; i++ ){
				cmap[i][0] = (float)(i-intrvl*3)/intrvl; 
				cmap[i][1] = 1; 
				cmap[i][2] = 0;
			}
			for( ; i<5*intrvl; i++ ){
				cmap[i][0] = 1; 
				cmap[i][1] = (float)(5*intrvl-i)/intrvl; 
				cmap[i][2] = 0;
			}
			for( ; i<n; i++ ) {
				cmap[i][0] = 1;
				cmap[i][1] = float(i-5*intrvl)/intrvl; 
				cmap[i][2] = float(i-5*intrvl)/intrvl;
			}
			break;
	}
}


void Colourscale :: colourize( float val )
{
	int indx=int(rint(a*val+b));
    if( indx<0 ) indx = 0;
    else if( indx>=n ) indx = n-1;
    glColor3fv( cmap[indx] );
}

void Colourscale :: size( int s )
{
	if( s<3 ) return;
	
	if( n ) {
		for( int i=0; i<n; i++ ) delete[] cmap[i];
		delete[] cmap;
	}

	n = s;
	cmap = new GLfloat* [n];
	for( int i=0; i<n; i++ ) cmap[i] = new GLfloat[3];

	calibrate( mindat, maxdat );
	scale( scaletype );
}

GLfloat* Colourscale :: colorvec( double val )
{
	int indx=int(rint(a*val+b));
    if( indx<0 ) indx = 0;
    else if( indx>=n ) indx = n-1;
    return cmap[indx];
}

// bounding box of colour bar 
const int lx=100, ux=200, ly=100, uy=600;

void
Colourscale :: eps( char *fn )
{
	ofstream out( fn );

	out << "%!PS-Adobe-2.0 EPSF-2.0\n";
	out << "%%Creator: flounder 0.1a Copyright 2000 Edward Vigmond\n";
	out << "%%Title: colorbar.eps\n";
	out << "%%BoundingBox: "<< lx << " "<< ly << " " << ux << " " << uy << endl;
	out << "%%EndComments\n";
	out << "/M { moveto } bind def /S { show  } bind def\n";
	out << "/R { rlineto } bind def /L { lineto } bind def\n";
	out << "/B { newpath 0 exch M 0 1 R 1 0 R 0 -1 R closepath fill } bind def\n";
	out << "gsave\n";
	out << lx << " " << ly << " translate\n";
	out << ux-lx<<" "<< (uy-ly)/float(n) << " scale\n";
	for( int i=0; i<n; i++ ) {
		out << cmap[i][0]<<" "<<cmap[i][1]<<" "<<cmap[i][2]<< " setrgbcolor\n";
		out << i << " B" << endl;
	}
	out << "showpage\ngrestore\n";
	out.close();
}
