/*
	flounder - copyright 2002 Edward J. Vigmond

	This file is part of flounder.     
	
	flounder is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.     
	
	flounder is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.     
	You should have received a copy of the GNU General Public License
    along with flounder; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
/*
 * Class for a colour map which returns a vector suitable for a glColor()
 * call
 */

#ifndef COLOURSCALE_H
#define COLOURSCALE_H
#include <GL/gl.h>
#include <math.h>
#include <iostream>
#include <fstream>
using namespace std;

typedef enum {
	CS_HOT, CS_GREY, CS_RAINBOW, CS_BL_RAINBOW, CS_COLD_HOT
} CScale_t;

class Colourscale {
	public:
		Colourscale(int ts=64, CScale_t tcs=CS_BL_RAINBOW );
		void     scale( CScale_t );				// set scale
		inline CScale_t scale(void){ return scaletype; }
		void     calibrate( double, double ); 	// set range
		inline float min(){return mindat;}		// return minimum data value
		inline float max(){return maxdat;}		// return maximum data value
		void colourize( float );				// colour vector for data value
		inline int size(){ return n; }			// get the size
		void     size( int );					// set the size
		inline GLfloat* entry(int a){return cmap[a];}	// return an entry
		GLfloat *colorvec( double );			// index matching data value
		void    eps( char * );					// output colour bar as EPS file
	private:
		GLfloat** cmap;							// the map
		int       n;							// size of map
		double    a, b;							// map data to colour map
		CScale_t  scaletype;
		double    mindat, maxdat;
};
		
#endif
	
