/*
	flounder - copyright 2002 Edward J. Vigmond

	This file is part of flounder.     
	
	flounder is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.     
	
	flounder is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.     
	You should have received a copy of the GNU General Public License
    along with flounder; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
//
// Gnuplot interface widget
// Copyrigth Jacques Tremblay jackt@gel.ulaval.ca
//
//
// This widget is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
//
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
// USA.
//

#include <stdlib.h>
#include <stdio.h>
#include <fstream.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include "FL/fl_draw.H"
#include "FL/fl_ask.H"
#include "FL/Fl_Menu_Item.H"
#include "FL/Fl_Window.H"
#include "Fl_Gnuplot.h"

#define N_ENTRIES 8192

// enter a GNUP{LOT command to be executed 
void gnuplotcom_cb( Fl_Widget *w, void *p )
{
	Fl_Gnuplot* graph=(Fl_Gnuplot*)w;

	char *s=fl_input("Enter Gnuplot commands seperated by ;",graph->command() );
	if( s != NULL ) graph->command(s);
	graph->window()->redraw();
}

Fl_Menu_Item graph_pop_menu[] = {
	{ "Gnuplot command", 'r', gnuplotcom_cb },
};

Fl_Gnuplot::Fl_Gnuplot(int x, int y, int w, int h, const char *l) :
  Fl_Box(x,y,w,h,l) {
  filename[0] = 0;
  loaded = 0;
  includeKey=0;
  scaleX = 1.0;
  scaleY = 1.0;

  Polar=0;

  currentIndex=0;

  nEntries = 0;
  nMaxEntries = N_ENTRIES;
  entries = new OutputEntry[nMaxEntries];

  loaded=0;
}

Fl_Gnuplot::~Fl_Gnuplot() {
  delete [] entries;
}

void
Fl_Gnuplot::clear_entries(int release) {
  nEntries = 0;
  if (!release || nMaxEntries <= N_ENTRIES) 
    return;
	cerr << "clear_entries\n";
  delete [] entries;
  nMaxEntries = N_ENTRIES;
  entries = new OutputEntry[nMaxEntries]; 
	cerr << "/clear_entries\n";
}

void
Fl_Gnuplot::add_entry(char *buf) {
  if (nEntries == nMaxEntries) {
    nMaxEntries += N_ENTRIES;
    OutputEntry *nn = new OutputEntry[nMaxEntries];
    int i;
    for(i=0 ; i<nEntries ; i++) {
      nn[i] = entries[i];
    }
    delete [] entries;
    entries = nn;
  }

	if (buf[0] == 'G') {
    clear_entries();
    nEntries = 1;
    entries[0].k = 'G';
		delete [] entries[0].t;
		entries[0].t = 0;
		return;
	}

  OutputEntry &e = entries[nEntries];
  nEntries++;
	
  char *str;
  e.k = buf[0];
	delete [] e.t;
  e.t = 0;

  switch(e.k) {
  case 'L':
    sscanf(buf, "L%4d", &e.x);
    e.x = (e.x % 8) + 2;
    break;
  case 'M':
    sscanf(buf, "M%4d%4d", &e.x, &e.y);
    break;
  case 'V':
    sscanf(buf, "V%4d%4d", &e.x, &e.y);
    break;
  case 'J':
    sscanf(buf, "J%4d", &e.x);
    break;
  case 'T':
    sscanf(buf, "T%4d%4d", &e.x, &e.y);
    str = buf + 9;
    e.t = new char[strlen(str)+1];
    strcpy(e.t, str);
    break;
    break;
  case 'R':
    break;
  }
  
}


enum JUSTIFY {
	LEFT, CENTRE, RIGHT
};

static int _colors[13] = { FL_WHITE, FL_BLACK, FL_BLACK, FL_BLACK,
			   FL_BLACK, FL_RED, FL_GREEN, FL_BLUE,
			   FL_MAGENTA, FL_CYAN, FL_GRAY, FL_GRAY, FL_GRAY };

void
Fl_Gnuplot::load() {
  int nread;
  int i,j;
  FILE *fifofptr,*gppipe;
  char fifofile[MAXPATHLEN];

  clear_entries(1);

  scaleX = 1/4096.0*w();
  scaleY = 1/4096.0*h();

  char buf[10*MAXPATHLEN];
  int tempchar;

  build_command(buf);
  gppipe = popen(buf, "w");
  setvbuf( gppipe, NULL, _IONBF, 0 );
  if (!gppipe) {
    printf("could not open pipe\n");
    return;
  }

  tmpnam(fifofile);
  if (mkfifo(fifofile,448)!=0) {
    fprintf(stderr,"Error creating FIFO pipe\n");
    exit (0);
  }


  if (GPcommand.size() != 0) {
    fprintf(gppipe,"%s\n",GPcommand.c_str());
  }

  fprintf(gppipe,"set term xlib\n");
  fprintf(gppipe,"set output \"%s\"\n",fifofile);
  if (Polar) {
    fprintf(gppipe,"set polar\n");
  }
  else {
    fprintf(gppipe,"set nopolar\n");
  }

  if (!includeKey) {
    fprintf(gppipe,"set nokey\n");
  }

  j=0;
  if (titles[j]) {
    fprintf(gppipe,"plot '-' title '%s' with lines",titles[j]);
  }
  else {
    fprintf(gppipe,"plot '-' with lines");
  }


  for (j=1;j<=currentIndex;j++) {
    if (titles[j]) {
      fprintf(gppipe,",'-' title '%s' with lines",titles[j]);
    }
    else {
      fprintf(gppipe,",'-' with lines");
    }
  }
  fprintf(gppipe,"\n");

  if ((fifofptr=fopen(fifofile,"rw"))==NULL) {
    fprintf(stderr,"Error opening FIFO file\n");
    exit(0);
  }

  for (j=0;j<=currentIndex;j++) {
    for(i=0 ; i<ndata[j] ; i++) {
      fprintf(gppipe,"%lf %lf\n",xdata[j][i],ydata[j][i]);
    }
    fprintf(gppipe,"e\n");
    fflush(gppipe);
  }

  do {
    if (fgets(buf, 1023, fifofptr)) {
      add_entry(buf);
    }
  } while (buf[0]!='E');

  fclose(fifofptr);
  unlink(fifofile);
  pclose(gppipe);
  loaded = 1;
}

void
Fl_Gnuplot::build_command(char *buf) {
  sprintf(buf, "gnuplot");
}

void
Fl_Gnuplot::play() const {
  char *str;
  double sw;
  int jmode;
  int cx, cy;
  int x1, y1;
  int lt;
  int i;

  fl_font(FL_TIMES, 12);
  fl_color(FL_BLACK);

  for(i=0 ; i<nEntries ; i++) { 
    OutputEntry &e = entries[i];
    switch(e.k) {
    case 'G':
      draw_box();
      draw_label();
      break;
    case 'L':
      lt = e.x;
      fl_color(_colors[lt+3]);
      break;
    case 'M':
      cx = e.x; cy = e.y;
      break;
    case 'V':
      x1 = e.x; y1 = e.y;
      fl_line(x() + x1*scaleX, y() + h()-y1*scaleY, x() + cx*scaleX, y() + h()-cy*scaleY);
      cx = x1;
      cy = y1;
      break;
    case 'J':
      jmode = e.x;
      break;
    case 'T':
      x1 = e.x; y1 = e.y;
      str = e.t;
      sw = fl_width(str);
      switch (jmode) {
      case LEFT:
	sw = 0;
	break;
      case CENTRE:
	sw = -sw / 2;
	break;
      case RIGHT:
	sw = -sw;
	break;
      }
      fl_color(_colors[2]);
      fl_draw(str, x() + x1*scaleX + sw, y() + h()-y1*scaleY + fl_height()/3);	   
      fl_color(_colors[lt+3]);
      break;
    case 'R': 
      break;
    }
  }
}

void
Fl_Gnuplot::draw() {
  if (!loaded)
    load();
  play();
}

void
Fl_Gnuplot::set_2d_data(const double *xx, const double *yy, 
												int n, const char *title) {		
        int i;
	currentIndex = 0;

	Polar=0;

	xdata=(double **)malloc((currentIndex+1)*sizeof(double *));
	ydata=(double **)malloc((currentIndex+1)*sizeof(double *));
	ndata=(int *)malloc((currentIndex+1)*sizeof(int));
	titles=(char **)malloc((currentIndex+1)*sizeof(char*));
	if (!xdata || !ydata || !ndata || !titles) {
	  fprintf(stderr,"Error allocating xdata or ydata or ndata titles arrays\n");exit(0);
	}

	xdata[currentIndex]=(double *)malloc(n*sizeof(double));
	ydata[currentIndex]=(double *)malloc(n*sizeof(double));
	if (!xdata[currentIndex] || !ydata[currentIndex]) {
	  fprintf(stderr,"Error allocating xdata[] or ydata[] arrays\n");exit(0);
	}
	titles[currentIndex]=(char *)malloc(MAXPATHLEN*sizeof(char));

	ndata[currentIndex]=n;
	for(i=0 ; i<ndata[currentIndex] ; i++) {
	  xdata[currentIndex][i]=xx[i];
	  ydata[currentIndex][i]=yy[i];
	}

	if (!title || !*title) {
	  titles[currentIndex]=NULL;
	}
	else if (title && *title) {
	  sprintf(titles[currentIndex],"%s",title);
	  includeKey=1;
	}

	loaded=0;
}

void
Fl_Gnuplot::add_2d_data(const double *xx, const double *yy, 
												int n, const char *title) {
        int i;
	currentIndex++;

	xdata=(double **)realloc(xdata,(currentIndex+1)*sizeof(double *));
	ydata=(double **)realloc(ydata,(currentIndex+1)*sizeof(double *));
	ndata=(int *)realloc(ndata,(currentIndex+1)*sizeof(int));
	titles=(char **)realloc(titles,(currentIndex+1)*sizeof(char *));
	if (!xdata || !ydata || !ndata || !titles) {
	  fprintf(stderr,"Error re-allocating xdata or ydata or ndata or titles arrays\n");exit(0);
	}

	xdata[currentIndex]=(double *)malloc(n*sizeof(double));
	ydata[currentIndex]=(double *)malloc(n*sizeof(double));
	if (!xdata[currentIndex] || !ydata[currentIndex]) {
	  fprintf(stderr,"Error allocating xdata[] or ydata[] arrays\n");exit(0);
	}
	titles[currentIndex]=(char *)malloc(MAXPATHLEN*sizeof(char));

	ndata[currentIndex]=n;
	for(i=0 ; i<ndata[currentIndex] ; i++) {
	  xdata[currentIndex][i]=xx[i];
	  ydata[currentIndex][i]=yy[i];
	}

	if (!title || !*title) {
	  titles[currentIndex]=NULL;
	}
	else if (title && *title) {
	  sprintf(titles[currentIndex],"%s",title);
	  includeKey=1;
	}

	loaded=0;

}


void
Fl_Gnuplot::set_2d_polar_data(const double *xx, const double *yy, 
															int n, const char *title) {
  
   


        int i;
	currentIndex = 0;

	Polar=1;

	xdata=(double **)malloc((currentIndex+1)*sizeof(double *));
	ydata=(double **)malloc((currentIndex+1)*sizeof(double *));
	ndata=(int *)malloc((currentIndex+1)*sizeof(int));
	titles=(char **)malloc((currentIndex+1)*sizeof(char*));
	if (!xdata || !ydata || !ndata || !titles) {
	  fprintf(stderr,"Error allocating xdata or ydata or ndata titles arrays\n");exit(0);
	}

	xdata[currentIndex]=(double *)malloc(n*sizeof(double));
	ydata[currentIndex]=(double *)malloc(n*sizeof(double));
	if (!xdata[currentIndex] || !ydata[currentIndex]) {
	  fprintf(stderr,"Error allocating xdata[] or ydata[] arrays\n");exit(0);
	}
	titles[currentIndex]=(char *)malloc(MAXPATHLEN*sizeof(char));

	ndata[currentIndex]=n;
	for(i=0 ; i<ndata[currentIndex] ; i++) {
	  xdata[currentIndex][i]=xx[i];
	  ydata[currentIndex][i]=yy[i];
	}

	if (!title || !*title) {
	  titles[currentIndex]=NULL;
	}
	else if (title && *title) {
	  sprintf(titles[currentIndex],"%s",title);
	  includeKey=1;
	}

	loaded=0;

}

void
Fl_Gnuplot::add_2d_polar_data(const double *xx, const double *yy, 
															int n, const char *title) {


        int i;
	currentIndex++;

	xdata=(double **)realloc(xdata,(currentIndex+1)*sizeof(double *));
	ydata=(double **)realloc(ydata,(currentIndex+1)*sizeof(double *));
	ndata=(int *)realloc(ndata,(currentIndex+1)*sizeof(int));
	titles=(char **)realloc(titles,(currentIndex+1)*sizeof(char *));
	if (!xdata || !ydata || !ndata || !titles) {
	  fprintf(stderr,"Error re-allocating xdata or ydata or ndata or titles arrays\n");exit(0);
	}

	xdata[currentIndex]=(double *)malloc(n*sizeof(double));
	ydata[currentIndex]=(double *)malloc(n*sizeof(double));
	if (!xdata[currentIndex] || !ydata[currentIndex]) {
	  fprintf(stderr,"Error allocating xdata[] or ydata[] arrays\n");exit(0);
	}
	titles[currentIndex]=(char *)malloc(MAXPATHLEN*sizeof(char));

	ndata[currentIndex]=n;
	for(i=0 ; i<ndata[currentIndex] ; i++) {
	  xdata[currentIndex][i]=xx[i];
	  ydata[currentIndex][i]=yy[i];
	}

	if (!title || !*title) {
	  titles[currentIndex]=NULL;
	}
	else if (title && *title) {
	  sprintf(titles[currentIndex],"%s",title);
	  includeKey=1;
	}

	loaded=0;

}


void
Fl_Gnuplot::resize(int _x, int _y, int _w, int _h) {
	Fl_Box::resize(_x, _y, _w, _h);
  scaleX = 1/4096.0*w();
  scaleY = 1/4096.0*h();
}

int
Fl_Gnuplot::handle( int event ) {
	if( event==FL_PUSH && Fl::event_button() == 3 ) {
		  const Fl_Menu_Item* m;
		  m = graph_pop_menu->popup(Fl::event_x(), Fl::event_y(), 0, 0, 0);
		  if (m) m->do_callback(this, (void*)m);
		  return 1;
	}
	return 0;
}

