/*
	flounder - copyright 2002 Edward J. Vigmond

	This file is part of flounder.     
	
	flounder is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.     
	
	flounder is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.     
	You should have received a copy of the GNU General Public License
    along with flounder; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "../config.h"
#include "flguppy.h"
#include <algorithm>
#ifdef HAVE_VALUES_H
#include <values.h>
#endif
#ifndef INT_MAX
#define INT_MAX (~0)
#endif
#include <sstream>

#ifdef HAVE_AVIFILE
#include <avifile/videoencoder.h>
#include <avifile/avifile.h>
#include <avifile/aviplay.h>
#include <avifile/utils.h>
#include <avifile/fourcc.h>
#endif

Fl_Font fontset[] = {
	FL_HELVETICA, FL_HELVETICA_BOLD, 
	FL_HELVETICA_ITALIC, FL_HELVETICA_BOLD_ITALIC,
	FL_COURIER, FL_COURIER_BOLD, FL_COURIER_ITALIC, FL_COURIER_BOLD_ITALIC,
	FL_TIMES, FL_TIMES_BOLD, FL_TIMES_ITALIC, FL_TIMES_BOLD_ITALIC,
	FL_SCREEN, FL_SCREEN_BOLD 
};

void setcurrent_cb( Fl_Widget *o, void *v );

void animate_cb( void *v )
{
	GlAreaUI *g = (GlAreaUI *)v;
	if( g->animDir == 0 ) return;
	g->time += int(g->animinc->value()*g->animDir);
	if( g->time<0 || g->time > g->tmslider->maximum() ) {
		switch( g->animmode->value() ) {
			case 0:						// just once
				g->time -= int(g->animinc->value()*g->animDir);
				g->animDir = 0;
				return;
				break;
			case 1:						// continuous
				g->time -= int(g->tmslider->maximum()*g->animDir);
				break;
			case 2:						// pendulum
				g->animDir *= -1;
				g->time += int(2*g->animinc->value()*g->animDir);
				break;
		}
	}
	g->tmslider->value(g->time);
	vector<GLView *>::iterator p=g->glwin.begin();
	while( p != g->glwin.end() ) {
		(*p)->model->update_time( g->time );
		p++;
	}
	Fl::add_timeout(g->animate_delay, animate_cb, v );
}

void GlAreaUI :: animate( int inc )
{
	animDir = inc;
	if( animmode->value() == 0 )	// once
		if( time+animinc->value()*animDir>tmslider->maximum() ||
			time+animinc->value()*animDir<0							)
			time -= int(tmslider->maximum()*animDir);
	Fl::add_timeout(0.01, animate_cb, this );
}

// increment time
void GlAreaUI :: tmstep( int inc )
{
	int posst = time+inc*int(animinc->value());
	if( posst >=0 && posst <= tmslider->maximum() ) {
		tmslider->value(posst);
		tmslider->redraw();
		time = posst;
		update_glwins();
	}
}

// request redraws for all glareas
void GlAreaUI :: update_glwins( )
{
	vector<GLView *>::iterator p=glwin.begin();
	while( p != glwin.end() ) {
		(*p)->model->update_time( time );
		p++;
	}
}


// add another model window
void  GlAreaUI :: add_glwin( char *fn=0 )
{
	glwin.resize(glwin.size()+1);
	glwin[glwin.size()-1] = new GLView( (GlAreaUI *)this, fn );
	if( glwin[glwin.size()-1]->model->data != NULL )
		setcurrent( glwin[glwin.size()-1] );
	else {
		glwin[glwin.size()-1]->~GLView();
		glwin.resize(glwin.size()-1);
		if( glwin.size()>0 )
			setcurrent( curr_glwin );
		else {
		    cerr << "Cannot find IGB file: " << fn << endl;
			exit( 1 );
		}
		return;
	}

	curr_glwin->timewin->but = tmbut;
	curr_glwin->horizon->but = horzbut;
	curr_glwin->vertical->but = vertbut;
	curr_glwin->model->change_font( fontset[fontchoice->value()],
										int(fontsizein->value()) );
	setimagemenu();
	set_tm_max();
}

//copy a model window
void GlAreaUI :: copy_glwin()
{
	glwin.resize(glwin.size()+1);
	glwin[glwin.size()-1] = new GLView( this, "", curr_glwin->model );
	setimagemenu();
	setcurrent( glwin[glwin.size()-1] );
}

// auto colur scale button
void GlAreaUI :: autocolbut( bool a )
{
	if( a==true ) {
		curr_glwin->model->autocol( true );
		minval->deactivate();
		maxval->deactivate();
		optimalbut->deactivate();
		autobutt->value( true );
	} else {
		curr_glwin->model->autocol( false );
		minval->value(curr_glwin->model->minval());
		maxval->value(curr_glwin->model->maxval());
		minval->activate();
		maxval->activate();
		optimalbut->activate();
		autobutt->value( false );
	}
}

//update the settings to reflect the current model window
void GlAreaUI :: setcurrent( GLView *a, bool force )
{
	if( a == curr_glwin && force==false ) return;

	if( find( glwin.begin(), glwin.end(), curr_glwin ) != glwin.end() ){
		curr_glwin->window->label(curr_glwin->model->fname.c_str());
		if( tabwin->window->visible() ) {
			curr_glwin->model->tableauoff = true;
			curr_glwin->model->redraw_overlay();
		}
	}

	//common
	vector<GLView *>::iterator p=glwin.begin();
	int i=0;
	while( *p++ != a ){
		i++;
	}
	imgmenu->value(i);
	curr_glwin = a;
	colscalech->value(a->model->colourscale());
	autocolbut(a->model->autocol());
	levels->value(float(curr_glwin->model->levels()));
	if(curr_glwin->model->bgd() )
		whitebgdbut->set();
	else
		whitebgdbut->clear();
	if( curr_glwin->model->showtm() )
		shtmbut->set();
	else
		shtmbut->clear();
	if(  curr_glwin->model->shading() )
		gourshadbut->set();
	else
		gourshadbut->clear();
	if( a->model->numdim() == 2 )
		tabs->value(group2d);
	else
		tabs->value(group3d);
	
	//2D
	switch( curr_glwin->model->dimdisp() ) {
		case DISP_XY:
		case DISP_YX:
			depthslide->maximum(curr_glwin->model->header.z());
			break;
		case DISP_YZ:
		case DISP_ZY:
			depthslide->maximum(curr_glwin->model->header.x());
			break;
		case DISP_ZX:
		case DISP_XZ:
			depthslide->maximum(curr_glwin->model->header.y());
			break;
	}
	depthslide->value(a->model->depth()+1);
	dimension->value(a->model->dimdisp());
	dispcursbut->value(a->model->cursor());
	curscolbut->color(a->model->cursor_colour());
	curscolbut->redraw();
	cursval->value(a->model->curval());
	curslines->value(a->model->curlines());
	curscoord->value(a->model->curcoord());
	cbarbut->value(a->model->colourbar());
	vertbut->value(a->vertical->window->shown());
	tmbut->value(a->timewin->window->shown());
	horzbut->value(a->horizon->window->shown());

	//3D
	xrotslide->value(a->model->xrot());
	xrotin->value(a->model->xrot());
	yrotslide->value(a->model->yrot());
	yrotin->value(a->model->yrot());
	zrotslide->value(a->model->zrot());
	zrotin->value(a->model->zrot());
	dispframebut->value(a->model->framedisp());
	dispaxesbut->value(a->model->axes());
	frcolourbut->color(a->model->frame_colour());
	frcolourbut->redraw();
	xmslide->maximum(curr_glwin->model->header.x());
	xpslide->maximum(curr_glwin->model->header.x());
	ymslide->maximum(curr_glwin->model->header.y());
	ypslide->maximum(curr_glwin->model->header.y());
	zmslide->maximum(curr_glwin->model->header.z());
	zpslide->maximum(curr_glwin->model->header.z());
	xmslide->value(curr_glwin->model->get_bb(BBXM) );
	xpslide->value(curr_glwin->model->get_bb(BBXP) );
	ymslide->value(curr_glwin->model->get_bb(BBYM) );
	ypslide->value(curr_glwin->model->get_bb(BBYP) );
	zmslide->value(curr_glwin->model->get_bb(BBZM) );
	zpslide->value(curr_glwin->model->get_bb(BBZP) );
	zscaleslide->value(a->model->zscale());
	zscaleout->value(a->model->zscale());

	//Isosurf tab
	GLfloat *col = curr_glwin->model->isosurf_colour( 0 );
	r0->value( col[0] );
	g0->value( col[1] );
	b0->value( col[2] );
	alpha0->value( col[3] );
	isovalin0->value( curr_glwin->model->isoval(0) );
	isobut0->value( curr_glwin->model->isosurf( 0 ) );
	col = curr_glwin->model->isosurf_colour( 1 );
	r1->value( col[0] );
	g1->value( col[1] );
	b1->value( col[2] );
	alpha1->value( col[3] );
	isovalin1->value( curr_glwin->model->isoval(1) );
	isobut1->value( curr_glwin->model->isosurf( 1 ) );

	//Contour tab
	bool sc, sd;
	int  nc;
	double st, inc;
	a->model->contours( sc, sd, nc, st, inc, true );
	contourbut->value( sc );
	shdatabut->value( sd );
	numcontslide->value( double( nc ) );
	stvalin->value( st );
	incvalin->value( inc );

	//Solid 3D tab
	solobjbut->value( a->model->solid3D()>0 );
	quickbut->value( a->model->solid3D()==1 );
	{
	  float min, max;
	  bool  rng = a->model->transrange( min, max );
	  transminval->value(min);
	  transmaxval->value(max);
	  transrangebut->value(rng);
	}
	rngcompbut->value(a->model->transrangecomp());

	// lights
	xlghtpos->value( a->model->lightpos('X'));
	ylghtpos->value( a->model->lightpos('Y'));
	zlghtpos->value( a->model->lightpos('Z'));
	lightbut->value( a->model->lights() );

	string wintitle = "** " + curr_glwin->model->fname;
	a->window->label(wintitle.c_str());
	a->window->show();
	help->update( curr_glwin);
	curr_glwin->model->tableau(tabwin);		// tableau limits
}


void GlAreaUI :: destroy_glwin( GLView *g )
{
	vector<GLView *>::iterator p=glwin.begin();
	while( *p != g ) p++;
	(*p)->~GLView();
	glwin.erase(p);
	setimagemenu();
	//exit if last window closed
	if( glwin.size() == 0 ) {
		//window->~Fl_Window();
		//return;
	}else {
	  // reset maximum allowable time
	  set_tm_max();
	  setcurrent( *(glwin.begin()) );
	}
}

GlAreaUI :: ~GlAreaUI()
{
	// this "if" is added because this function could be called
	// implicitly after being called explicitly
	if( window != NULL ) {
		vector<GLView *>::iterator p=glwin.begin();
		while( p != glwin.end() ) {
			(*p++)->~GLView();
		}
		glwin.~vector();
		window->~Fl_Window();
		window = NULL;
	}
}

//time series button calback - toggle display of window
void GlAreaUI :: timebutcb( bool a )
{
	if( a == true ){
		int n=int(tmslider->value());
		double *t;
		double *data=curr_glwin->model->timevec(n, t);
		curr_glwin->timewin->window->show();
		curr_glwin->timewin->set_data(n,data,int(this->tmslider->value()),t);
	}else
		curr_glwin->timewin->window->hide();
}

//horizontal profile button callback
void GlAreaUI :: horzbutcb( bool a )
{
	if( a == true ){
		int n;
		double *data=curr_glwin->model->horizontalvec(n);
		curr_glwin->horizon->window->show();
		curr_glwin->horizon->set_data(n,data);
	}else
		curr_glwin->horizon->window->hide();
}

//vertical profile button callback
void GlAreaUI :: vertbutcb( bool a )
{
	if( a == true ){
		int n;
		double *data=curr_glwin->model->verticalvec(n);
		curr_glwin->vertical->window->show();
		curr_glwin->vertical->set_data(n,data);
	}else
		curr_glwin->vertical->window->hide();
}

// select current model from choice button
void GlAreaUI :: setimagemenu()
{
	//update image choice menu
	imgmenu->clear();
	imgmenu->user_data( (void *)this );
	int j=0;
	for( vector<GLView *>::iterator p=glwin.begin(); p<glwin.end(); p++, j++ ) {
		int nummatch;
		nummatch = 0;
		for( int i=0; i<j; i++ ){
			char menutit[256];
			strcpy( menutit, imgmenu->text(i) );
			if( menutit[strlen(menutit)-1] == ')' ){
				int l=strlen(menutit)-3;
				while( menutit[l] != '(' ) l--;
				menutit[l] = '\0';
			}
			if( (*p)->model->menuname == menutit ) nummatch++;
		}
		
		if( !nummatch )
			imgmenu->value( imgmenu->add((*p)->model->menuname.c_str(), 0,
						setcurrent_cb, *p, 0) );
		else {
			ostringstream ost;
			//add the trailing '\0' for the C string requirement
			ost << (*p)->model->menuname << "(" << nummatch << ")" << '\0';
			imgmenu->value( imgmenu->add(ost.str().c_str(), 0, setcurrent_cb, *p, 0) );
		}
	}

}

//find greatest common time
void GlAreaUI :: set_tm_max()
{
	vector<GLView *>::iterator p=glwin.begin();
	int mint = (*p)->model->header.t();
	while( ++p != glwin.end() )
		if( (*p)->model->header.t()<mint ) mint=(*p)->model->header.t();
	if( tmslider->maximum() > mint-1 ){
		tmslider->value(mint-1);
		p=glwin.begin();
		while( ++p != glwin.end() )
			(*p)->model->update_time(mint-1);
	}
	tmslider->maximum(mint-1);
	tmslider->redraw();
}

inline int square( int a ){return a*a; }

// make gif images
void GlAreaUI :: movie( bool single )
{
	char *fn;
	if( single == true )
		fn = fl_file_chooser("Choose output filename", "*.gif", "image.gif" );
	else
		fn = fl_file_chooser("Choose output filename", "*.gif", "movie.gif" );
	if( fn == NULL ) return;

	window->cursor(FL_CURSOR_WAIT);
	curr_glwin->window->cursor(FL_CURSOR_WAIT);

	double tmsave = tmslider->value();
	int width = curr_glwin->window->w();
	int height = curr_glwin->window->h();
  	GLubyte* buffer = new GLubyte[width*height*4];
	GifPixelType* tableind = new GifPixelType[width*height];
	glReadBuffer(GL_BACK);

	// get current colour map
	ColorMapObject* gifcmap = MakeMapObject( 256, NULL );
	int cmap_size = curr_glwin->model->gif_cmap( gifcmap );

	EGifSetGifVersion( "89a" );
	GifFileType *gft = EGifOpenFileName( fn, 0 );
	EGifPutScreenDesc( gft, width, height, 256, 0,  gifcmap );

	// application block for Netscape infinte looping 
	// if cancel is pressed, do not do looping
	unsigned short tmdelay = 0;
	if( single == false ) {
		const char* tmdelaystr = fl_input("Enter time delay (1/100's of a second)", "0");
		if( tmdelaystr != NULL ) {
			tmdelay = atoi(tmdelaystr);
			char*  applext = new char[11];
			strcpy( applext, "NETSCAPE2.0" );
			EGifPutExtensionFirst( gft, 0xff, 11, applext );
			applext[0] = 1;
			applext[1] = 0;
			applext[2] = 0;
			EGifPutExtensionLast( gft, 0x21, 3, applext );
			delete[] applext;
		}
	}
	
	// transparent background and time delay
	unsigned char *grphext = new unsigned char[4];
	if( curr_glwin->model->numdim()==3 || curr_glwin->model->colourbar()==true
			|| curr_glwin->model->header.transparent() != NULL )
		if( fl_ask("Do you want the background transparent?") ) {
			grphext[0] = 0x09; 			//transparent, disposal method=2
			grphext[3] = 0;
		} else {
			grphext[0] = 0x00;
			grphext[3] = 0;
		}
	grphext[1] = tmdelay&0xff;
	grphext[2] = tmdelay<<8;
	
	for( int tt=int(tmslider->value());tt<=tmslider->maximum(); tt+=int(animinc->value())){

		curr_glwin->model->update_time(tt);	//set the time
		tmslider->value(tt);
		tmslider->redraw();
		Fl::flush();						//update window

		// read buffer and match colours to colour map
  		glReadPixels(0,0,width,height,GL_RGB,GL_UNSIGNED_BYTE,(GLvoid *)buffer);
		for( int i=0; i<3*width*height; i+=3 ) {
			unsigned int dist, mindist=INT_MAX, closest;
			for( int j=0; j<cmap_size; j++ ) {
				dist = square(buffer[i] - gifcmap->Colors[j].Red) + 
					   square(buffer[i+1]-gifcmap->Colors[j].Green) + 
					   square(buffer[i+2]-gifcmap->Colors[j].Blue);
				if( dist<mindist ) {
					closest = j;
					if( !(mindist = dist) ) break;
				}
			}
			tableind[(((i/3)%width)+(height-1-(i/3)/width)*width)] = closest;
		}
		//output one image
		EGifPutExtension( gft, 0xf9, 4, grphext );
		if( EGifPutImageDesc( gft, 0, 0, width, height, FALSE, NULL )==GIF_ERROR )
			cout << "GIF problem with image header" << endl;
		if( EGifPutLine( gft, tableind, width*height )==GIF_ERROR )
			cout << "GIF problem writing pixels" << endl;
		if( single == true ) break;			// only do single image 
	}
	EGifCloseFile( gft );
  	delete[] buffer;
	delete[] tableind;
	delete[] grphext;
	FreeMapObject( gifcmap );
	tmslider->value(int(tmsave));
	curr_glwin->model->update_time(int(tmslider->value()));
	window->cursor(FL_CURSOR_DEFAULT);
	curr_glwin->window->cursor(FL_CURSOR_DEFAULT);
}


// make avi images
void 
GlAreaUI :: avi_movie( void )
{
#ifndef HAVE_AVIFILE
  cerr << "You must have avifile installed under Linux for this to work\n";
  return;
#else

	char *fn = fl_file_chooser("Choose output filename", "*.avi", "movie.avi" );
	if( fn == NULL ) return;

	window->cursor(FL_CURSOR_WAIT);
	curr_glwin->window->cursor(FL_CURSOR_WAIT);

	double tmsave = tmslider->value();
	int width = curr_glwin->window->w();
	int height = curr_glwin->window->h();
  	GLubyte* buffer = new GLubyte[width*height*4];
	glReadBuffer(GL_BACK);

	// initialize AVI stream
	int fccHandler;
	int codec=fl_choice( "Choose a codec:", "I263", "IV32", "CVID" ); 
	switch( codec ) {
		case 0: fccHandler = fccI263;
				break;
		case 1: fccHandler = fccIV32;
				break;
		case 2: fccHandler = fccCVID;
				break;
	}
	IAviWriteFile* avifile=CreateIAviWriteFile(fn);

	// repeat frames to make movie longer
	int numpad=0;
	const char *nps=fl_input( "Number of padding frames", "0" );
	if( nps==NULL )
		return;
	else {
		if( (numpad=atoi( nps ))<0 )
			numpad = 0;
	}
		
	// initialize a BMP header
	BITMAPINFOHEADER bh;
	memset( &bh, 0, sizeof(BITMAPINFOHEADER) );
	bh.biSize = 40;
	bh.biWidth  = width;
	bh.biHeight = height;
	bh.biPlanes = 1;
	bh.biBitCount = 24;
	bh.biCompression = 0;
	bh.biSizeImage = width*height*3;
	bh.biXPelsPerMeter = 2913;
	bh.biYPelsPerMeter = 2913;
	bh.biClrUsed = 0;
	bh.biClrImportant = 0;
	IAviVideoWriteStream* stream=avifile->AddVideoStream(fccHandler, &bh, 100000);
	stream->SetQuality(8000);
	CImage im((const BitmapInfo*)&bh, (const uint8_t* )buffer, false);	
	stream->Start();

	for( int tt=int(tmslider->value());tt<=tmslider->maximum(); tt+=int(animinc->value())){

		curr_glwin->model->update_time(tt);	//set the time
		tmslider->value(tt);
		tmslider->redraw();
		Fl::flush();						//update window

  		glReadPixels(0,0,width,height,GL_RGB,GL_UNSIGNED_BYTE,(GLvoid *)buffer);
		
		// swap bytes 1 and 3 because the order is actually BGR in the BMP
		// file?
		for( int i=0; i<width*height*3; i+=3 ) {
		  int tmp = buffer[i];
		  buffer[i] = buffer[i+2];
		  buffer[i+2] = tmp;
		}
		for( int i=0; i<1+numpad; i++ )
			stream->AddFrame(&im);
	}
	stream->Stop();
	delete avifile;
  	delete[] buffer;
	
	tmslider->value(int(tmsave));
	curr_glwin->model->update_time(int(tmslider->value()));
	window->cursor(FL_CURSOR_DEFAULT);
	curr_glwin->window->cursor(FL_CURSOR_DEFAULT);
#endif
}


//set font
void GlAreaUI :: setfont()
{
	vector<GLView *>::iterator p=glwin.begin();
	while( p != glwin.end() ) {
		(*p)->model->change_font( fontset[fontchoice->value()], 
								  int(fontsizein->value())		 );
		p++;
	}
}

//choose colour for isosurfaces
#include <FL/Fl_Color_Chooser.H>
void GlAreaUI:: change_isocolour( int c, Fl_Value_Input* rw, 
				Fl_Value_Input* gw,	Fl_Value_Input *bw, double alpha )
{
	double r = rw->value();
	double g = gw->value();
	double b = bw->value();
	if( fl_color_chooser( "Isosurface colour", r, g, b ) ){
		rw->value( r );
		gw->value( g );
		bw->value( b );
		curr_glwin->model->isosurf_colour( c, r, g, b, alpha );
	}
}
	

void GlAreaUI::set_contour()
{
	bool cb = bool(contourbut->value());
	bool sd = bool(shdatabut->value());
	int  nl = int(numcontslide->value());
	double st = stvalin->value();
	double inc = incvalin->value();
	curr_glwin->model->contours( cb, sd, nl, st, inc );
}

void GlAreaUI::output_eps()
{
	char *fn=fl_file_chooser("Output EPS file", "*.eps", "image.eps" );
	if( fn==NULL ) return;

	RenderEPS* reo = new RenderEPS( fn ); 
	curr_glwin->model->renderAsEPS( reo );
	reo->~RenderEPS();
}

#include "PNGwrite.h"
void GlAreaUI::output_png()
{
	char *fn=fl_file_chooser("Output PNG file", "*.png", "image.png" );
	if( fn==NULL ) return;
	FILE *out;
	if( (out=fopen(fn,"w")) == NULL )
	  return;

	PNGwrite* pngimg = new PNGwrite( out );
	int width =  curr_glwin->window->w();
	int height = curr_glwin->window->h();
	pngimg->size( width, height );
	pngimg->depth( 8*sizeof(GLubyte) );
	pngimg->colour_type( PNG_COLOR_TYPE_RGB_ALPHA );
	
  	GLubyte* buffer = new GLubyte[width*height*4];
	glReadBuffer(GL_BACK);
	curr_glwin->model->make_current();
  	glReadPixels(0,0,width,height,GL_RGBA,GL_UNSIGNED_BYTE,(GLvoid *)buffer);
	if( curr_glwin->model->numdim()==3 || curr_glwin->model->colourbar()==true
			|| curr_glwin->model->header.transparent() != NULL )
		if( !fl_ask("Do you want the background transparent?") ) {
		  GLubyte bgdcol[] = {0,0,1};
		  for( int i=0; i<width*height; i++ )
			if( !memcmp( bgdcol, buffer+i*4, 3*sizeof(GLubyte) ) )
			  buffer[i*4+3] = 255;
		}
	pngimg->write( buffer );
	delete[] buffer;
	delete pngimg;
}

