/*
	flounder - copyright 2002 Edward J. Vigmond

	This file is part of flounder.     
	
	flounder is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.     
	
	flounder is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.     
	You should have received a copy of the GNU General Public License
    along with flounder; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include <FL/gl.h>
#include <FL/Fl_Gl_Window.H>
#include "IGBheader.h"
#include <cstdio>
#include <cstdlib>
#include <string.h>
#include <vector>
//#include <strstream>
#include <sstream>
#include <fstream>
extern "C" {
#include <gif_lib.h>
}
#include <FL/fl_file_chooser.H>
#include <FL/fl_ask.H>
#include <FL/fl_draw.H>
#include <FL/fl_show_colormap.H>
#include <FL/Fl_Chart.H>
#include "zlib.h"
#include "flguppy.h"
#ifdef HAVE_LIBGRACE_NP
#include "grace_np.h"
#endif
#include "isovis.h"
#include "Colourscale.h"
#include "RenderEPS.h"	
#include "IGBheader.h"

typedef enum dispdim{DISP_XY, DISP_YZ, DISP_ZX, 
								DISP_YX, DISP_ZY, DISP_XZ} DispDim; 
typedef enum bbvalue{ BBXM, BBXP, BBYM, BBYP, BBZM, BBZP } BBvalue;

const int numisosurf = 2;

class GLView;

class Glarea : public Fl_Gl_Window {

	int  datumSize;			//size of data type
	float **slice;			// 2D slice data
	int  xd, yd;			// dimensions of 2D slices
	int  maxdim;			// maximum dimension in x, y, z
	int  numDim;			// #dimensions to display
	int  currTime;			// time to display
	int  currDepth;			// current value of nondisplayed 2D dimension
	int  maxDepth;			// max for non displayed dimension in 2D
	DispDim dim_disp;		// dimesions to display in 2D
	bool swapbytes;
	bool autocs;				
	bool disphoriz;
	bool dispvert;
	bool disptime;
	bool gouraudshade;
	bool dispcursor;
	bool displines;
	bool dispvalue;
	bool dispcoord;
	bool disptitle;
	bool dispcolourbar;
	bool dispaxes;
	bool showtime;
	short xrotate, yrotate, zrotate;
	int bbxm, bbxp, bbym, bbyp, bbzm, bbzp;			 //3D bounding box
	float zscalefac;					//factor by which to scale z dimension
	bool disp3Dframe;
	bool disp3Daxes;
	GLfloat bgdcolour[3];		// background colour
	GLfloat cursorColour[3];
	int flcursorcolour;			// fltk colour map index for cursorColour
	GLfloat frameColour[3];
	int flframecolour;			// fltk colour map index for frameColour
	int xcursor, ycursor;
	void dataslice( int, int );
	void draw();
	void draw_overlay();
	int handle(int);
	float getdatum( int, int, int, int );
	float getdatum( int, int, int, int, float& );
	// transform from raw to actual data value
	inline float toval( float a ) { return a/header.facteur()+header.zero(); }
	// convert user value to raw transformed value
	inline float toraw( float a ) { return (a-header.zero())*header.facteur(); }
	TableauUI* tabui;
	Fl_Font fontface;
	int   fontsize;
	bool  showiso[numisosurf];			// true to show isovalue surfaces
	float isovalue[numisosurf];			// value of isosurface
	GLfloat isocol[numisosurf][4];		// colour of isosurfaces
	bool  showcontour;					// true to show contours
	bool  showdata;						// true to show data with contours
	int   numcontour;					// number of contour lines
	float contourval, contourinc;		// contour starting val and increment
	int   show3Dsolid;					// true to draw solid objects
	float trans_min, trans_max;			// range of transparent data
	bool  rngcomp;						// use complement of transparent range
	bool  transparent( int,int,int,int );// determine if pixel transparent
	bool  lightson;						// lights on for 3D effects
	float xlightpos, ylightpos, zlightpos; // positions of lights
	Colourscale *cscale;				// colour scale object
	bool  feedingback;					// true if in feedbackmode
	void determine_visible_verts(void); // make list of verts
	int  num_visible_vert;
	int*  vis_vert_list;

public:
	Glarea(int X, int Y, int W, int H, const char *L=0);
	~Glarea();
	GLView *pwin;							// parent window
	void* data;								// data 
	int*  datalinks;						// number of links to data 
	string fname;							// full filename
	string menuname;						// name in menu
	IGBheader header;						// IGB header
	int  getdata( string );					// read in new IGB file
	void sharedata( void *d ){data = d;}	// create new view of existing data
	int  reread();							// reread file, only t changed
	void numdim( const char *, const bool );// set #dimensions to display
	inline int numdim(void){return numDim;}	// get # dimensions displayed
	void update_time( int t );				// set time to display
	void depth( int d );					// set depth to display
	int  depth();							// get displayed depth
	void framedisp( int );					// set display of frame in 3D
	inline int framedisp(void){ return  disp3Dframe;}// frame displayed in 3D?
	void frame_colour( int );				// set frame FLTK colour index
	int  frame_colour();					// get frame colour
	void cursor_colour( int );				// set cursor FLTK colour index
	int  cursor_colour();					// get cursor colour
	void bgd( int );						// set background white or black
	int  bgd();								// get background colour
	void axes( int );						// set aces displayed
	int  axes();							// get axes displayed
	void  xrot( float );					// set x rotation
	inline float xrot(){ return xrotate; }	// get x rotation
	void  yrot( float );					// set y rotation
	inline float yrot(){ return yrotate; }	// get y rotation
	void  zrot( float );					// set z rotation
	inline float zrot(){ return zrotate; }	// get z rotation
	void  zscale( float );					// set factor to scale z dimension
	inline float zscale(){return zscalefac;}// get z scaling factor
	void update_bb( BBvalue, double );		// update the 3D bounding box
	int  get_bb( BBvalue );				// get bounding box 
	void colourscale( CScale_t );		// set colour scale map
	inline CScale_t colourscale(void){return cscale->scale();}	// get colour scale map
	void calibratecolour( void );		// calibrate colour scale for frame
	void 	dimdisp( DispDim );			// set dimensions displayed in 2D
	DispDim dimdisp();					// dimensions displayed in 2D
	void minval( double );				// set minimum data value
	double minval( void );				// minimum data value
	void maxval( double );				// set maximum data value
	double maxval( void );				// maximum data value
	void autocol( bool );				// set autocalibrate on/off
	bool autocol();						// calibrate colour scale for each frame
	void curlines( bool );				// set diplay of cursor lines on/off
	bool curlines();					// display of cursor lines
	void curval( bool );				// set display of value under cursor
	bool curval();						// display of value under cursor
	void curcoord( bool );				// set display of cursor coord on/off
	bool curcoord();					// display of cursor coordinates
	void cursor( bool );				// set display of cursor
	bool cursor();						// cursor displayed or not
	void colourbar( bool );				// set colour bbar on/off
	bool colourbar();					// status of colour bar
	double *timevec( int &, double* & );// get time series from cursor position
	double *horizontalvec( int & );		// get horizontal profile
	double *verticalvec( int & );		// get vertical profile
	void cursor( int, int );			// set cursor position
	void shading( bool );				// true for Gouraud shading
	bool shading();						// Gouraud shading?
	void copy( Glarea *b );				// copy but share data
	void swap_bytes();					// swap byte order
	void tableau( TableauUI *tb, bool a=0 );
	void plot_tableau(TableauUI *);
	void tableau_area( TableauUI * );
	bool tableauoff;
	void levels( int );					// number of colour map entries
	inline int  levels( void ){ return cscale->size(); }
	int gif_cmap( ColorMapObject * ); 	//fill in GIF cmap from current cmap
	void showtm( bool );				// show current time on image
	inline bool showtm( void ){ return showtime; }
	void change_font( Fl_Font, int );	// set the display font
	void isosurf( int s, bool a ){showiso[s]=a; valid(0); redraw();}
	inline bool isosurf( int s ){return showiso[s]; }
	inline void isoval(int s, float a){ isovalue[s]=toraw(a);redraw(); }
	inline float isoval( int s ){return toval(isovalue[s]); }
	void isosurf_colour( int, double, double, double, double );
	GLfloat* isosurf_colour( int );
	void contours( bool&, bool&, int&, double&, double&, bool update=false );
	void subsample();					// subsample the current image data
	inline int solid3D(){ return show3Dsolid; } // solid3D on or off?
	void solid3D( int a );
	bool transrange( float& min, float &max );	// return transparency range
	void transrange( double min, double max, bool range );
    inline bool transrangecomp(){ return rngcomp; }      // range complemented?
	void transrangecomp( bool a ){ rngcomp=a; redraw(); }//complement range
	void lightpos( char, float );		// set light position
	float lightpos( char );				// get light position
	void lights( bool a ){ lightson = a; valid(0); redraw(); }
	bool lights( void ){ return lightson; }
	void renderAsEPS( RenderEPS *r );
	inline void output_cbar( void ){ cscale->eps( "colourbar.eps" ); }
	void colourize( float& );			
};
