/*
	flounder - copyright 2002 Edward J. Vigmond

	This file is part of flounder.     
	
	flounder is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.     
	
	flounder is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.     
	You should have received a copy of the GNU General Public License
    along with flounder; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
//
// "$Id: arc.cxx,v 1.4.2.2 2000/06/05 21:21:15 mike Exp $"
//
// Arc drawing test program for the Fast Light Tool Kit (FLTK).
// class Graph
//
// a class for drawing simple 2D plots on linear x and y scales (for now)
// 
#include "Graph.h"
#include <FL/Fl_Menu_Item.H>
#include <FL/fl_ask.H>
#include <string>
#include "flguppy.h"

const int setcolour[]={FL_RED, FL_GREEN, FL_BLUE, FL_BLACK, 
       FL_YELLOW , FL_MAGENTA, FL_CYAN, FL_WHITE, FL_SELECTION_COLOR, 
   															   FL_GRAY}; 
const float l_marg=0.1;
const float r_marg=0.05;
const float top_marg=.05;
const float bot_marg=0.1;
const float l_axis_sep=0.;
const float tic_length=0.025;
const float plotw=1.-r_marg-l_marg;
const float ploth=1.-top_marg-bot_marg;

// input an x-range to display
void xrange_cb( Fl_Widget *w, void *p )
{
	Graph* g = (Graph *)w;
	double a, b, c, d;
	g->range( a, b, c, d );
	char range[256];
	sprintf( range, "%f:%f", a, b );
	const char *s=fl_input("Enter X range (x1:x2)", range );
	if( s != NULL && sscanf( s, "%lf : %lf", &a, &b ) == 2 ){
		g->set_range( a, b, c, d );
		g->redraw();
	}
}

// input an y-range to display
void yrange_cb( Fl_Widget *w, void *p )
{
	Graph* g = (Graph *)w;
	double a, b, c, d;
	g->range( a, b, c, d );
	char range[256];
	sprintf( range, "%f:%f", c, d );
	const char *s=fl_input("Enter Y range (y1:y2)", range );
	if( s != NULL && sscanf( s, "%lf : %lf", &c, &d ) == 2 ){
		g->set_range( a, b, c, d );
		g->redraw();
	}
}

void reset_cb( Fl_Widget *w, void *p )
{
	((Graph *)w)->reset_view();
}

const int button_height=25;

void
but_disp_toggle_cb( Fl_Widget *w, void *p )
{
  PlotWin *pwin=(PlotWin *)(((Graph *)w)->window()->user_data());

  if( pwin->button_grp->visible()){
  	pwin->button_grp->hide();
	pwin->graph->size(pwin->graph->w(),pwin->graph->h()+button_height );
	pwin->window->redraw();
  }else {
	pwin->graph->size(pwin->graph->w(),pwin->graph->h()-button_height );
  	pwin->button_grp->show();
	pwin->window->redraw();
  }
}


void
close_graph_cb( Fl_Widget *w, void *p )
{
  PlotWin *pwin=(PlotWin *)(((Graph *)w)->window()->user_data());
  
  pwin->window->hide();
  pwin->but->value(0);
}

Fl_Menu_Item graph_pop_menu[] = {
	{ "reset view", 'r', reset_cb },
	{ "X range", 'x', xrange_cb },
	{ "Y range", 'y', yrange_cb },
	{ "toggle button display", 'b', but_disp_toggle_cb },
	{ "close", 'c', close_graph_cb }
};


// plot the entire data set
void Graph::reset_view(void) {
	x0=xmin;x1=xmax;y0=ymin;y1=ymax;
	make_labels();
	redraw();
}


// given the zoom window coordinates, set the new range to plot
void Graph::change_view( int nx, int ny, int nw, int nh )
{
	int    xx0 = int(x()+l_marg*w());
	double  dx = w()*plotw;
	double x0n = x0 + (x1-x0)*float(nx-xx0)/dx;
	double x1n = x0 + (x1-x0)*float(nx+nw-xx0)/dx;

	int    yy0 = int(y()+(1.-bot_marg)*h());
	double  dy = h()*ploth;
	double y0n = y0 + (y1-y0)*float(yy0-ny-nh)/dy;
	double y1n = y0 + (y1-y0)*float(yy0-ny)/dy;

	set_range( x0n, x1n, y0n, y1n );
	redraw();
}

// main drawing routine
void Graph :: draw() {
	
	fl_push_matrix();
    fl_color(230,230,230);			// grey background
    fl_rectf(x(),y(),w(),h());
	// determine drawing transformation
	fl_translate( x(), y() );
	fl_scale( plotw*w()/(x1-x0), -ploth*h()/(y1-y0) );
	fl_translate( -x0+(x1-x0)*l_marg/plotw, -y1-(y1-y0)*top_marg/ploth );
	// draw the curves
	fl_push_clip( x()+int(w()*l_marg), y()+int(h()*top_marg), 
									int(w()*plotw), int(h()*ploth) );
	for( int i=0; i<numset; i++ ){
		fl_color( setcolour[i] );
		fl_begin_line();
		for( int j=0; j<np[i]; j++ )
			fl_vertex( xv[i][j], yv[i][j] );
		fl_end_line();
	}
	fl_pop_clip();
	// draw frame
	fl_color(FL_BLACK );
	fl_begin_line();
	fl_vertex( x0, y0 );
	fl_vertex( x1, y0 );
	fl_vertex( x1, y1 );
	fl_vertex( x0, y1 );
	fl_vertex( x0, y0 );
	fl_end_line();
	// draw labels
	fl_font( FL_HELVETICA, 10 );
	for( int i=0; i<num_labels; i++ ) {
		// x labels
		fl_draw( xlabel[i],
			int(x()+float(w())*(l_marg+i*plotw/nlm1)-fl_width(xlabel[i])/2.),
														int(y()+0.97*h()) );
		// y labels
		fl_draw( ylabel[i], x(),
			int(y()+h()*(1.-0.1/1.15-1./1.15*i/nlm1)),
							int(l_marg*w()), 0, (Fl_Align)(FL_ALIGN_RIGHT) );
		// botom x tics
		fl_begin_line();
			fl_vertex( x0+(x1-x0)*i/nlm1, y0 );
			fl_vertex( x0+(x1-x0)*i/nlm1, y0+(y1-y0)*tic_length );
		fl_end_line();
		// top x tics
		fl_begin_line();
			fl_vertex( x0+(x1-x0)*i/nlm1, y0+(y1-y0) );
			fl_vertex( x0+(x1-x0)*i/nlm1, y0+(y1-y0)*(1.-tic_length) );
		fl_end_line();
		// left y tics
		fl_begin_line();
		fl_begin_line();
			fl_vertex( x0, y0+(y1-y0)*i/nlm1 );
			fl_vertex( x0+(x1-x0)*tic_length, y0+(y1-y0)*i/nlm1 );
		fl_end_line();
		// right y tics
		fl_begin_line();
			fl_vertex( x1, y0+(y1-y0)*i/nlm1 );
			fl_vertex( x0+(x1-x0)*(1.-tic_length), y0+(y1-y0)*i/nlm1 );
		fl_end_line();
	}
	fl_pop_matrix();
}


/*
 * reset the plotting widget and add the first set to plot
 */
int 
Graph :: set_2d_data( const double *x, const double *y, int n )
{
	numset = 0;
    return add_2d_data( x, y, n );
}


/*
 * add another set to plot
 */
int 
Graph :: add_2d_data( const double *x, const double *y, int n )
{
  	if( numset == max_num_sets-1 )
		return 1;

	xv[numset] = x;
	yv[numset] = y;
	np[numset] = n;

	if( numset==0 ) {
		xmin = xmax = x[0];
		ymin = ymax = y[1];
	}
	
	for( int i=0; i<n; i++ ){
		if( x[i]<xmin )
			xmin = x[i];
		if( x[i]>xmax )
			xmax = x[i];
		if( y[i]<ymin )
			ymin = y[i];
		if( y[i]>ymax )
			ymax = y[i];
	}
	x0 = xmin;
	x1 = xmax;
	y0 = ymin;
	y1 = ymax;
	
	numset++;
	make_labels();
	redraw();

	return 0;
}


// generate the string labels for the axes
void Graph::make_labels( void ){
	
	for( int i=0; i<num_labels; i++ ){
	  sprintf( xlabel[i], "%.4g", x0 + (x1-x0)*i/((float)num_labels-1.));
	  sprintf( ylabel[i], "%.4g", y0 + (y1-y0)*i/((float)num_labels-1.));
	}

}

int Graph::handle(int event) {
  static int ix, iy;
  static int dragged;
  static int sx, sy, sw, sh;

  switch (event) {
  case FL_PUSH:
	  if( Fl::event_button() == 1 ) {
		  ix = Fl::event_x(); if (ix<x()) ix=x(); if (ix>=x()+w()) ix=x()+w()-1;
		  iy = Fl::event_y(); if (iy<y()) iy=y(); if (iy>=y()+h()) iy=y()+h()-1;
		  dragged = 0;
		  return 1;
	  } else if( Fl::event_button() == 3 ) {
		  const Fl_Menu_Item* m;
		  m = graph_pop_menu->popup(Fl::event_x(), Fl::event_y(), 0, 0, 0);
		  if (m) m->do_callback(this, (void*)m);
		  return 1;
	  }
	break;
  case FL_DRAG:
    dragged = 1;
    int x2,y2;
    x2 = Fl::event_x(); if (x2<x()) x2=x(); if (x2>=x()+w()) x2=x()+w()-1;
    y2 = Fl::event_y(); if (y2<y()) y2=y(); if (y2>=y()+h()) y2=y()+h()-1;
    if (ix < x2) {sx = ix; sw = x2-ix;} else {sx = x2; sw = ix-x2;}
    if (iy < y2) {sy = iy; sh = y2-iy;} else {sy = y2; sh = iy-y2;}
    window()->make_current();
    fl_overlay_rect(sx,sy,sw,sh);
    return 1;
	break;
  case FL_RELEASE:
    if ( Fl::event_button() != 1 || !dragged) 
		return 0;
	dragged = 0;
	window()->make_current();
	fl_overlay_clear();
    if ( sw < 3 || sh < 3) 
		return 0;
	else
		change_view( sx, sy, sw, sh );
  }
  return 0;
}

// return the range currently plotted
void Graph::range( double& a, double& b, double& c, double& d )
{
	a = x0; b = x1; c = y0; d = y1;
}

// set the range to plot
void Graph::set_range( double a, double b, double c, double d ) 
{
	if( x1>x0 ) {
		x0 = a; x1 = b;
	} else {
		x0 = b; x1 = a;
	}
	if( y1>y0 ) {
		y0 = c; y1 = d;
	} else {
		y0 = d; y1 = c;
	}
	make_labels();
}
