/*
	flounder - copyright 2002 Edward J. Vigmond

	This file is part of flounder.     
	
	flounder is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.     
	
	flounder is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.     
	You should have received a copy of the GNU General Public License
    along with flounder; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#define CONTOUR_MAIN
#include <contour.h>

const int array_inc = 1000;
int       array_size = 0;


/*
 * generate contours form a table of table
 *
 * use a 2D marching squares algorithm
 */
void Contour::generate_contour( float **data, int nx, int ny, double value )
{
	num_vert = 0;

	for( int i=0; i<nx-1; i++ )
		for( int j=0; j<ny-1; j++ ) {
			short scenario = (data[i][j]>value) + (data[i+1][j]>value)*2 + 
						 	(data[i+1][j+1]>value)*4 + (data[i][j+1]>value)*8;
			
			num_vert += 2;
			if( num_vert >= array_size ) {
				array_size += array_inc;
				vx = (GLfloat *)realloc( vx, array_size*sizeof(GLfloat) );
				vy = (GLfloat *)realloc( vy, array_size*sizeof(GLfloat) );
			}

			switch( scenario ) {
				case 0:
				case 15:
					num_vert -= 2;
					break;
				case 1:
				case 14:
					vx[num_vert-2] = i;
					vy[num_vert-2] = j + 
								(value-data[i][j])/(data[i][j+1]-data[i][j]);
					vx[num_vert-1] = i +
								(value-data[i][j])/(data[i+1][j]-data[i][j]);
					vy[num_vert-1] = j;
					break;
				case 2:
				case 13:
					vx[num_vert-2] = i +
								(value-data[i][j])/(data[i+1][j]-data[i][j]);
					vy[num_vert-2] = j;
					vx[num_vert-1] = i+1;
					vy[num_vert-1] = j + (value-data[i+1][j])/
										(data[i+1][j+1]-data[i+1][j]);
					break;
				case 3:
				case 12:
					vx[num_vert-2] = i;
					vy[num_vert-2] = j + (value-data[i][j])/
										(data[i][j+1]-data[i][j]);
					vx[num_vert-1] = i+1;
					vy[num_vert-1] = j + (value-data[i+1][j])/
										(data[i+1][j+1]-data[i+1][j]);
					break;
				case 4:
				case 11:
					vx[num_vert-2] = i+1;
					vy[num_vert-2] = j + (value-data[i+1][j])/
										(data[i+1][j+1]-data[i+1][j]);
					vx[num_vert-1] = i + (value-data[i][j+1])/
										(data[i+1][j+1]-data[i][j+1]);
					vy[num_vert-1] = j+1;
					break;
				case 5:
				case 10:
					num_vert += 2;
					if( num_vert >= array_size ) {
						array_size += array_inc;
						vx = (GLfloat *)realloc( vx, array_size*sizeof(GLfloat) );
						vy = (GLfloat *)realloc( vy, array_size*sizeof(GLfloat) );
					}
					vx[num_vert-4] = i+1;
					vy[num_vert-4] = j + (value-data[i+1][j])/
										(data[i+1][j+1]-data[i+1][j]);
					vx[num_vert-3] = i + (value-data[i][j+1])/
										(data[i+1][j+1]-data[i][j+1]);
					vy[num_vert-3] = j+1;
					vx[num_vert-2] = i;
					vy[num_vert-2] = j + 
								(value-data[i][j])/(data[i][j+1]-data[i][j]);
					vx[num_vert-1] = i +
								(value-data[i][j])/(data[i+1][j]-data[i][j]);
					vy[num_vert-1] = j;
					break;
				case 6:
				case 9:
					vx[num_vert-2] = i + (value-data[i][j])/
							(data[i+1][j] - data[i][j] );
					vy[num_vert-2] = j;
					vx[num_vert-1] = i + (value-data[i][j+1])/
							(data[i+1][j+1] - data[i][j+1] );
					vy[num_vert-1] = j+1;
					break;
				case 7:
				case 8:
					vx[num_vert-2] = i;
					vy[num_vert-2] = j + (value-data[i][j])/
											(data[i][j+1]-data[i][j]);
					vx[num_vert-1] = i + (value-data[i][j+1])/
											(data[i+1][j+1]-data[i][j+1]);
					vy[num_vert-1]  = j+1;
					break;
			}
		}
}
