/*
	flounder - copyright 2002 Edward J. Vigmond

	This file is part of flounder.     
	
	flounder is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.     
	
	flounder is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.     
	You should have received a copy of the GNU General Public License
    along with flounder; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/* file_io.c               -Brian Tierney,  LBL   12/90  */

/* This file contains code for read HIPS files to be used with isovis,
   as well as the routine to output ASCII and XGL formated vertex lists
 */

/*
* Portions developed at the National Center for Supercomputing Applications at
* the University of Illinois at Urbana-Champaign.
*/

#define MAIN

#include <unistd.h>
#include <stdlib.h>
#include "isovis.h"
#include <math.h>
#include <stdio.h>
#include <string.h>

const char      *Progname = "isovis";	// used by error routine 
const char      *MY_NAME="Isosurf";
int    			 xmax, ymax, zmax;		// xmax = xdim -1, etc 
CUBE_TRIANGLES **prevslice, **currslice;
int    			 xdim, ydim, zdim;		// size of input data volume 
Grid_type     ***grid;					// segmentation grid 
FILE            *poly_fp = NULL;		// polygon output file 


using namespace Isosurf;

#ifdef CRAY
char  *cray_convert_to_int_and_pack(float *px, float *py, float *pz, float *nx, 
						float *ny, float *nz, int nverts, int size);
#endif

void tessellate_volume( float l_tval );
void write_poly_header();
void init_header(struct header *hd, char *onm, char *snm, int nfr, 
			char *odt, int rw, int cl, int bpp, int bpk, int pfmt, char *desc);
void update_header(struct header *hd, int argc, char **argv);
void fwrite_header(int fd, struct header *hd);
//fileio.c
int write_polys_ascii(float *px, float *py, float *pz, float *nx, 
					float *ny, float *nz, int nverts, int* conn, int nconn);
int write_polys_xgl(float *px, float *py, float *pz, float *nx, float *ny,
											float *nz, int nverts, int more);
int xgl_write_buffer(char *outbuf_ptr, int write_size);
char *vertex_list_to_xyz_order(float *px, float *py, float *pz, float *nx, 
							float *ny, float *nz, int nverts, int size);
int write_polys_scry(float *px, float *py, float *pz, float *nx, float *ny,
					float *nz, int nverts, int* conn, int nconn);
//hips.cc
void read_header(struct header *hd);
int  fread_headerc(int fd, header *hd, int eflag);
//iso.c
int iso_surface(int startx, int starty, int endx, int endy, int chunk_loc, 
		int slice, float threshold);
int calc_index_and_temps(int x1, int y1, int gridz, int dataz, float threshold,
							float DATA[9]);
int get_cell_verts(int index, float x1, float y1, float z1, float threshold, 
					float crossings[13][3], float DATA[9] );
void get_cell_polys(int index, int *npolys, float crossings[13][3], int x, 
														int y, int z);
//misc.c
void   alloc_3d_grid_array(int nx, int ny, int nz);	
float    *alloc_list(unsigned int size);
float    *realloc_list(unsigned int size, float *buf);
int      *conn_alloc(unsigned int size, int *buf);
CUBE_TRIANGLES **alloc_2d_cube_array(int nx, int ny);
float   **alloc_2d_float_array(int nx, int ny);
void free_3d_grid_array();
int free_2d_float_array(float   **array);
// poly.c
int add_polygon(float *p1, float *p2, float *p3, int x, int y, int z);
int search_for_dup(float vert[3], int num_to_check, int conn_idx);
//segment.c
int seg_slice_thresh(float thresh, float **data2d, Grid_type **grid2d, 
		int sx, int sy, int ex, int ey,  int slice);
int flood_fill_segmentation(int startx, int starty, int endx, int endy, 
		int slice, int chunk_size);
void push(int i, int j, int k);
void pop(int *i, int *j, int *k);
void exit_handler();


/**************************** do everything ***********************/
int 
Isosurf::make_isosurface( int xd, int yd, int zd, float lower_thresh )
{
    num_vertices = num_conn = 0;
	if( xvertices != NULL ) {
		free(xvertices);
		free(yvertices);
		free(zvertices);
		xvertices = yvertices = zvertices = NULL;
		if( xnormals != NULL ) {
			free(xnormals);
			free(ynormals);
			free(znormals);
			xnormals = ynormals = znormals = NULL;
		}
		if( connectivity != NULL ) {
			free(connectivity);
			connectivity = NULL;
		}
	}
	xdim = xd;
	ydim = yd;
	zdim = zd;
	if (grid != NULL) free_3d_grid_array();
	alloc_3d_grid_array(CHUNK_SIZE, ydim, xdim);
	tessellate_volume( lower_thresh );
}

/*
 * alloc_3d_data_array - allocate a triply subscripted array to
 * 						 hold the image data
 * 						 N.B. the array is subscripted as data[z][y][x]
 */
void
Isosurf::alloc_3d_data_array(int nx, int ny, int nz, float *dp )
{
    register int i, j;

	data = Calloc( nz, float ** );
	if( dp == NULL ){
    	if ( (dp = Calloc(nx * ny * nz, float)) == NULL)
			perror("calloc error: Isosurf::data array ");
	} 
	for( i=0; i<nz; i++ ){
		data[i] = Calloc( ny, float * );
		for( j=0; j<ny; j++ )
			data[i][j] = dp + nx*(i*ny + j);
	}
	return;
		
    /* allocate 3-d array for input image data */

    /* allocate 2 arrays of pointers */
    if ((Isosurf::data = Calloc(nx, float **)) == NULL)
	perror("calloc error: Isosurf::data array ");
    if ((Isosurf::data[0] = Calloc(nx * ny, float *)) == NULL)
	perror("calloc error: Isosurf::data array ");

    /* allocate array for Isosurf::data */
	if( dp == NULL ){
    	if ((Isosurf::data[0][0] = Calloc(nx * ny * nz, float)) == NULL)
			perror("calloc error: Isosurf::data array ");
	} else
		Isosurf::data[0][0] = dp;

    /* initialize pointer arrays */
    for (i = 1; i < ny; i++)
		Isosurf::data[0][i] = Isosurf::data[0][0] + (nz * i);
    for (i = 1; i < nx; i++) {
		Isosurf::data[i] = Isosurf::data[0] + (ny * i);
		Isosurf::data[i][0] = Isosurf::data[0][0] + (ny * nz * i);
		for (j = 1; j < ny; j++)/* initialize pointer array */
	    	Isosurf::data[i][j] = Isosurf::data[i][0] + (nz * j);
    }
}

/****************** free_3d_data_array  *****************************/
void
Isosurf::free_3d_data_array(int keep_data)
{
	if( !keep_data ) free((char *) data[0][0]);
	for( int i=0; i<zdim; i++ )
    	free( data[i]);
    free( data);
}

		
/**************************** write_polys ****************************/

int
write_polys_ascii(float *px, float *py, float *pz, float *nx, float *ny, float *nz, int nverts, int* conn, int nconn)
/* This subroutine writes out an ascii file of the polygons */
{

    register int i, j;
    int       nc;

    fprintf(stderr, "\nWriting %d vertices and %d edges.\n", nverts, nconn);

    fprintf(poly_fp, "nverts: %d, nconn: %d\n", nverts, nconn);

    /* write out all of the vertices */
    for (i = 0; i < nverts; i++) {
	if (fprintf(poly_fp, "vert: %.3f %.3f %.3f \n",
		    px[i], py[i], pz[i]) == -1) {
	    fprintf(stderr, "%s: error, can't write to file \n",
		    MY_NAME);
	}
    }

    if (DO_NORMALS) {
	for (i = 0; i < nverts; i++) {
	    if (fprintf(poly_fp, "norm: %.3f %.3f %.3f \n",
			nx[i], ny[i], nz[i]) == -1) {
		fprintf(stderr, "%s: error, can't write to file \n",
			MY_NAME);
	    }
	}
    }
    /*
     * And the list of polygon sides.
     */
    j = 0;
    nc = nconn / 3;
    for (i = 0; i < nc; i++)
	fprintf(poly_fp, "edge: %d %d %d \n",
		conn[j++], conn[j++], -(conn[j++]));

    return 0;
}

/***************************************************************/
int
write_polys_xgl(float *px, float *py, float *pz, float *nx, float *ny, float *nz, int nverts, int more)
/* This subroutine writes out an xgl polygon file */
{
    int       write_size, ival;
    int       buf[2];		/* header output buffer */
    char     *outbuf_ptr;
    float     elapsed_time;
#ifdef CRAY
#define DHZ 166722200.0		/* PSC Cray Y-MP clock time = 5.998 ns; DHZ =
				 * 1/clock */
    struct tms tbuf;
    long      start_time, end_time;
#else
    struct timeb start_time, end_time;
#endif

    /*
     * Header: more flag:  1 = MAY be more vertices to follow nverts: number
     * of vertices
     */

    fprintf(stderr, "\nWriting %d vertices.\n", nverts);
#ifdef SPEED
#ifdef CRAY
    start_time = times(&tbuf);
#else
    ftime(&start_time);
#endif
#endif


    Fwrite(&more, sizeof(int), 1, poly_fp);
    Fwrite(&nverts, sizeof(int), 1, poly_fp);

    if (nverts <= 0)
	return 0;

#ifdef CRAY
    nverts += nverts % 2;	/* force nverts to be an even number */
    write_size = (nverts * 3) / 2;
    if (DO_NORMALS)
	write_size = write_size * 2;

    outbuf_ptr = cray_convert_to_int_and_pack(px, py, pz, nx, ny, nz,
					      nverts, write_size);
    write_size = write_size * sizeof(int);

#else
    write_size = nverts * 3;
    if (DO_NORMALS)
	write_size = write_size * 2;

    outbuf_ptr = vertex_list_to_xyz_order(px, py, pz, nx, ny, nz,
					  nverts, write_size);
    write_size = write_size * sizeof(float);
#endif
    xgl_write_buffer(outbuf_ptr, write_size);

#ifdef SPEED
#ifdef CRAY
    end_time = times(&tbuf);
    fprintf(stderr, "\nTime for writing vertices to file/socket: %f seconds\n",
	    (end_time - start_time) / DHZ);
#else
    ftime(&end_time);
    elapsed_time = ((float) ((end_time.time * 1000) + end_time.millitm) -
		    ((start_time.time * 1000) + start_time.millitm)) / 1000.;
    fprintf(stderr, "\nTime for writing vertices to file/socket: %f seconds\n",
	    elapsed_time);
#endif
#endif

    return 0;
}

/***********************************************************/
int
xgl_write_buffer(char *outbuf_ptr, int write_size)
{

    if (write(fileno(poly_fp), outbuf_ptr, write_size) != write_size)
	perror("error writing output_buf");
    return 0;
}

/****************************************************************/
char     *
vertex_list_to_xyz_order(float *px, float *py, float *pz, float *nx, 
							float *ny, float *nz, int nverts, int size)
 /* combines separate arrays of x,y, and z vertices into one buffer */
{
    static float *output_buf = NULL;
    static int last_write_size;
    register int i, j;

    if (output_buf == NULL) {
	if ((output_buf = Calloc(size, float)) == NULL)
	    perror("calloc: output_buf");
    } else if (size > last_write_size) {
	if ((output_buf = (float *) realloc((char *) output_buf,
					    size * sizeof(float))) == NULL) {
	    perror("realloc output_buf");
	    exit_handler();
	}
    }
    last_write_size = size;

    j = 0;
    for (i = 0; i < nverts; i++) {
	output_buf[j++] = px[i];
	output_buf[j++] = py[i];
	output_buf[j++] = pz[i];
	if (DO_NORMALS) {
	    output_buf[j++] = nx[i];
	    output_buf[j++] = ny[i];
	    output_buf[j++] = nz[i];
	}
    }

    return ((char *) output_buf);
}

/*****************************************************************/
#ifdef CRAY
char     *
cray_convert_to_int_and_pack(float *px, float *py, float *pz, float *nx, 
						float *ny, float *nz, int nverts, int size)
{
    static int *int_output_buf = NULL;
    static int last_write_size;
    int       int_scale;
    register int i, j;
    unsigned int ival;

    if (int_output_buf == NULL) {
	if ((int_output_buf = Calloc(size, int)) == NULL)
	    perror("calloc: int_output_buf");
    } else if (size > last_write_size) {
	if ((int_output_buf = (int *) realloc((char *) int_output_buf,
					      size * sizeof(int))) == NULL) {
	    perror("realloc int_output_buf");
	    exit_handler();
	}
    }
    last_write_size = size;

    /* scale all values to ints between 0 and LARGE_INT */
    int_scale = LARGE_INT / MAX_DIM;

    if ((nverts % 2) != 0)
	px[nverts] = py[nverts] = pz[nverts] = 0.0;

    j = 0;

    if (!DO_NORMALS) {
#ifdef CRAY
#pragma _CRI ivdep
#endif
	for (i = 0; i < nverts; i += 2) {

	    /* pack x,y into one word */
	    ival = (unsigned) (*px * int_scale);
	    ival <<= 32;
	    ival |= (unsigned) (*py * int_scale);
	    int_output_buf[j++] = ival;
	    px++;
	    py++;

	    /* pack z,x into one word */
	    ival = (unsigned) (*pz * int_scale);
	    ival <<= 32;
	    ival |= (unsigned) (*px * int_scale);
	    int_output_buf[j++] = ival;
	    pz++;
	    px++;

	    /* pack y,z into one word */
	    ival = (unsigned) (*py * int_scale);
	    ival <<= 32;
	    ival |= (unsigned) (*pz * int_scale);
	    int_output_buf[j++] = ival;
	    py++;
	    pz++;

	}
    } else {			/* with normals */

#ifdef CRAY
#pragma _CRI ivdep
#endif
	for (i = 0; i < nverts; i++) {

	    /* pack x,y into one word */
	    ival = (unsigned) (*px * int_scale);
	    ival <<= 32;
	    ival |= (unsigned) (*py * int_scale);
	    int_output_buf[j++] = ival;
	    px++;
	    py++;

	    /* pack z,nx into one word */
	    ival = (unsigned) (*pz * int_scale);
	    ival <<= 32;
	    ival |= (unsigned) ((int) (*nx * int_scale) & 0x00000000FFFFFFFF);
	    int_output_buf[j++] = ival;
	    pz++;
	    nx++;

	    /* pack ny,nz into one word */
	    ival = (unsigned) ((int) (*ny * int_scale) & 0x00000000FFFFFFFF);
	    ival <<= 32;
	    ival |= (unsigned) ((int) (*nz * int_scale) & 0x00000000FFFFFFFF);
	    int_output_buf[j++] = ival;
	    ny++;
	    nz++;
	}
    }
    return ((char *) int_output_buf);
}
#endif
/***************************************************************/
int
write_polys_scry(float *px, float *py, float *pz, float *nx, float *ny,
					float *nz, int nverts, int* conn, int nconn)
 /* This subroutine writes out a scry polygons file */
{
    register int i, j;
    int       n, nc, vrt;

    fprintf(stderr, "SCRY output: Writing %d vertices and %d edges.\n",
	    nverts, nconn);

    /*
     * write num vertices, num edges, transparency, and vertex normal
     * indicator
     */
    fwrite(&nverts, sizeof(int), 1, poly_fp);
    fwrite(&nconn, sizeof(int), 1, poly_fp);
    n = 10;			/* no transparency */
    fwrite(&n, sizeof(int), 1, poly_fp);
    if (DO_NORMALS)
	n = 1;
    else
	n = 0;			/* no normals */
    fwrite(&n, sizeof(int), 1, poly_fp);

    /* write vertices  */

    fwrite(px, sizeof(float), nverts, poly_fp);	/* write all x values */
    fwrite(py, sizeof(float), nverts, poly_fp);	/* write all y values */
    fwrite(pz, sizeof(float), nverts, poly_fp);	/* write all z values */
    if (DO_NORMALS) {
	fwrite(nx, sizeof(float), nverts, poly_fp);	/* write all x normals */
	fwrite(ny, sizeof(float), nverts, poly_fp);	/* write all y normals */
	fwrite(nz, sizeof(float), nverts, poly_fp);	/* write all z normals */
    }
    /* write edges  */

    j = 0;
    nc = nconn / 3;
    for (i = 0; i < nc; i++) {
	vrt = conn[j++] + 1;	/* add one because SCRY expects vertex count
				 * to start at 1 */
	fwrite(&vrt, sizeof(int), 1, poly_fp);
	vrt = conn[j++] + 1;
	fwrite(&vrt, sizeof(int), 1, poly_fp);

	vrt = -(conn[j++] + 1);	/* last item in list is negated */
	fwrite(&vrt, sizeof(int), 1, poly_fp);
    }
    return 0;
}

/**************************************************************/
void
exit_handler()
{
    fprintf(stderr, "Exiting... ");
    exit(-1);
}
/*	Copyright (c) 1989 Michael Landy

Disclaimer:  No guarantees of performance accompany this software,
nor is any responsibility assumed on the part of the authors.  All the
software has been tested extensively and every effort has been made to
insure its reliability.   */

/*
 * read_header.c - HIPL Picture Format Header read
 *
 * Michael Landy - 2/1/82
 * modified to use read/write 4/26/82
 */


#define LINES 100
#define LINELENGTH 132

static char *ssave[LINES];
static int slmax[LINES];
static int lalloc = 0;

void fread_header(int fd, struct header *hd);
int  perr(char *s, int a1=0, int a2=0, int a3=0, int a4=0, int a5=0, int a6=0);
char *desc_massage(char *s);
char    *halloc(int i, int j);
void write_header(struct header *hd);
int wstr(int fd, char *s);
int wstrlen(char *s);
int wnocr(int fd, char *s);
int dfprintf(int fd, int i);
/**************************************************************/

int perr(char *s, int a1, int a2, int a3, int a4, int a5, int a6)
{
    fprintf(stderr, "%s: ", Progname);
    fprintf(stderr, s, a1, a2, a3, a4, a5, a6);
    fprintf(stderr, "\n");
    exit(1);
}

/******************************************************/

/*
 * halloc.c - HIPL Picture Format core allocation.
 *
 * Michael Landy 2/1/82
 */

char     *
halloc(int i, int j)
{
    char     *k;

    k = (char *) calloc(i, j);
    if (k == NULL)
	perr("Not enough core available.");
    return (k);
}


/********************************************************/

/*
 * desc_massage.c - HIPL Picture Format description massager.
 *
 * Michael Landy - 2/2/82
 */

char     *
desc_massage(char *s)
{
    int       len, i, k, beg;

    len = strlen(s);
    beg = 1;
    for (i = 0; i < len; i++) {
	if (beg && *(s + i) == '.' && *(s + i + 1) == '\n') {
	    for (k = i + 2; k <= len; k++)
		*(s + k - 2) = *(s + k);
	    i--;
	    len -= 2;
	} else if (s[i] == '\n')
	    beg = 1;
	else
	    beg = 0;
    }
    return (s);
}

/*********************************************************************/

/*******************************************************************/

/*
 * wsubs.c - HIPL image header write header subroutines
 *
 * Michael Landy - 2/1/82
 * modified to use read/write - 4/26/82
 * modified to return #chars - msl - 9/21/90
 */

int 
wstr(int fd, char *s)
{
    int       i;

    i = strlen(s);
    write(fd, s, i);
    if (s[0] == '\0' || s[i - 1] != '\n') {
	write(fd, "\n", 1);
	i++;
    }
    return (i);
}

int 
wstrlen(char *s)
{
    int       i;

    i = strlen(s);
    if (s[0] == '\0' || s[i - 1] != '\n')
	i++;
    return (i);
}

int 
wnocr(int fd, char *s)
{
    char     *t;
    int       i;

    t = s;
    i = 0;
    while (*t != '\n' && *t != '\0') {
	write(fd, (t++), 1);
	i++;
    }
    write(fd, "\n", 1);
    return (i + 1);
}

int 
dfprintf(int fd, int i)
{
    char      s[50];
    int       j;

    sprintf(s, "%d\n", i);
    j = strlen(s);
    write(fd, s, j);
    return (j);
}

/* iso.c    routine for computing the vertex locations  */

/*
 * This program implements the marching cubes surface tiler described by
 * Lorensen & Cline in the Siggraph 87 Conference Proceedings.
 *
 * Original program written by Mike Krogh, NCSA, Feb.  2, 1990
 * Portions developed at the National Center for Supercomputing Applications at
 * the University of Illinois at Urbana-Champaign.
 *
 * modified by Brian Tierney, LBL  12/90
 *            Lawrence Berkeley Laboratory
 *            Imaging Technologies Group
 *            email: bltierney@lbl.gov
 *
 *  changes include removal of SGI Iris specific code, and various
 *  speed enhancements.
*/

#include "cell_table.h"

/**************************** iso_surface ****************************/

int
iso_surface(int startx, int starty, int endx, int endy, int chunk_loc, 
		int slice, float threshold)
/* This subroutine will generate the polygonal isosurface. */
{
    register int x, y, xdim1, ydim1;
    int       index;
    int       npolys;
    CUBE_TRIANGLES **tmpslice;
    int       skip_cnt = 0, skip;
    float     crossings[13][3];
	float     DATA[9];

    npolys = 0;			/* keep count of total polygons */

    for (y = 0; y < 3; y++)	/* initialize triangles array */
	for (x = 0; x < 13; x++)
	    crossings[x][y] = 0.;

    xdim1 = endx - 1;
    ydim1 = endy - 1;

    skip_cnt = 0;
    for (y = starty; y < ydim1; y++) {
	for (x = startx; x < xdim1; x++) {
	    index = calc_index_and_temps(x, y, chunk_loc, slice,
					 threshold, DATA );

	    if (index < 0) {	/* index < 0 means can skip the next index
				 * cubes */
		skip = -index;
		skip_cnt += skip;
		x += skip;
	    } else if (index > 0) {	/* index of 0 means no triangles */

		if (DUP_CHECK) {/* initialize */
		    currslice[y][x].edge_index = -1;
		    currslice[y][x].num_edges = 0;
		}
		if ((get_cell_verts(index, (float) x, (float) y,
				  (float) slice, threshold, crossings, DATA)) < 0)
		    return (-1);
		get_cell_polys(index, &npolys, crossings, x, y,
			       slice);
	    }
	}
    }
    if (DUP_CHECK) {
	tmpslice = currslice;
	currslice = prevslice;
	prevslice = tmpslice;
    }
    if (VERBOSE)
	fprintf(stderr, "slice %d: %d polygons \n", slice, npolys);

    if (SKIP_HOLES && VERBOSE)
	fprintf(stderr, "skipped %d cubes by segmentation pre-pass\n", skip_cnt);

    return (npolys);
}

/************************* calc_index_and_temps ****************************/

int
calc_index_and_temps(int x1, int y1, int gridz, int dataz, float threshold,
										float DATA[9] )

/* This subroutine calculates the index into the cell_table,
 * and creates some global temporary variables (for speed).
 */
{
    int       x2, y2, gz2, dz2, index = 0, rval = 0;
    Grid_type grid_cube[8];
    register int i;
    float     nval;

    x2 = x1 + 1;
    y2 = y1 + 1;
    gz2 = gridz + 1;
    dz2 = dataz + 1;

    grid_cube[0] = grid[gridz][y1][x1];
    grid_cube[1] = grid[gridz][y1][x2];
    grid_cube[2] = grid[gridz][y2][x2];
    grid_cube[3] = grid[gridz][y2][x1];
    grid_cube[4] = grid[gz2][y1][x1];
    grid_cube[5] = grid[gz2][y1][x2];
    grid_cube[6] = grid[gz2][y2][x2];
    grid_cube[7] = grid[gz2][y2][x1];


    if (grid_cube[0] > 0)
	index++;
    if (grid_cube[1] > 0)
	index += 2;
    if (grid_cube[2] > 0)
	index += 4;
    if (grid_cube[3] > 0)
	index += 8;

    if (grid_cube[4] > 0)
	index += 16;
    if (grid_cube[5] > 0)
	index += 32;
    if (grid_cube[6] > 0)
	index += 64;
    if (grid_cube[7] > 0)
	index += 128;

    if (SKIP_HOLES) {
	rval = -9999;
	for (i = 0; i < 8; i++) {
	    rval = rval>grid_cube[i] ? rval : grid_cube[i];
	}
    }
    if (index == 0) {
	if (SKIP_HOLES && rval != -9999)
	    return (rval);
	else
	    return (0);
    }
    if (index == 255)		/* no triangles in this cube */
	return (0);


    /*
     * DATA[1] thru DATA[8] are the 8 values in the original data which make up
     * the cube
     */

    DATA[1] = data[dataz][y1][x1];
    DATA[2] = data[dataz][y1][x2];
    DATA[3] = data[dataz][y2][x2];
    DATA[4] = data[dataz][y2][x1];

    DATA[5] = data[dz2][y1][x1];
    DATA[6] = data[dz2][y1][x2];
    DATA[7] = data[dz2][y2][x2];
    DATA[8] = data[dz2][y2][x1];

    if (SEG_METHOD == 1 || SMALLER_BOX == 1) {
	/* create false value N % from the thresh value */
	//nval = threshold - ((data_max - data_min) * .1);
	nval = threshold;

/*	nval = threshold; */

	if (DATA[1] > threshold && grid_cube[0] <= 0)
	    DATA[1] = nval;

	if (DATA[2] > threshold && grid_cube[1] <= 0)
	    DATA[2] = nval;

	if (DATA[3] > threshold && grid_cube[2] <= 0)
	    DATA[3] = nval;

	if (DATA[4] > threshold && grid_cube[3] <= 0)
	    DATA[4] = nval;

	if (DATA[5] > threshold && grid_cube[4] <= 0)
	    DATA[5] = nval;

	if (DATA[6] > threshold && grid_cube[5] <= 0)
	    DATA[6] = nval;

	if (DATA[7] > threshold && grid_cube[6] <= 0)
	    DATA[7] = nval;

	if (DATA[8] > threshold && grid_cube[7] <= 0)
	    DATA[8] = nval;
    }
    return (index);
}

/**************************** get_cell_verts ****************************/

int
get_cell_verts(int index, float x1, float y1, float z1, float threshold, 
					float crossings[13][3], float DATA[9] )
{
/* This routine computes the vertex locations */

    register int i;
    float     x2, y2, z2, val;
    int       nedges;
    int       crnt_edge;

#define linterp(d1,d2,t,x) ((t-d1) / ((d2-d1)) + x)

    x2 = x1 + 1.;
    y2 = y1 + 1.;
    z2 = z1 + 1.;

    nedges = cell_table[index].nedges;
    for (i = 0; i < nedges; i++) {
	crnt_edge = cell_table[index].edges[i];
	switch (crnt_edge) {
	case 1:
	    if (DATA[2] == DATA[1])
		val = x1;
	    else {
		val = linterp(DATA[1], DATA[2], threshold, x1);
	    }
	    crossings[1][0] = val;
	    crossings[1][1] = y1;
	    crossings[1][2] = z1;
	    break;

	case 2:
	    if (DATA[3] == DATA[2])
		val = y1;
	    else {
		val = linterp(DATA[2], DATA[3], threshold, y1);
	    }
	    crossings[2][1] = val;
	    crossings[2][0] = x2;
	    crossings[2][2] = z1;
	    break;

	case 3:
	    if (DATA[3] == DATA[4])
		val = x1;
	    else {
		val = linterp(DATA[4], DATA[3], threshold, x1);
	    }
	    crossings[3][0] = val;
	    crossings[3][1] = y2;
	    crossings[3][2] = z1;
	    break;

	case 4:
	    if (DATA[4] == DATA[1])
		val = y1;
	    else {
		val = linterp(DATA[1], DATA[4], threshold, y1);
	    }
	    crossings[4][1] = val;
	    crossings[4][0] = x1;
	    crossings[4][2] = z1;
	    break;

	case 5:
	    if (DATA[6] == DATA[5])
		val = x1;
	    else {
		val = linterp(DATA[5], DATA[6], threshold, x1);
	    }
	    crossings[5][0] = val;
	    crossings[5][1] = y1;
	    crossings[5][2] = z2;
	    break;

	case 6:
	    if (DATA[7] == DATA[6])
		val = y1;
	    else {
		val = linterp(DATA[6], DATA[7], threshold, y1);
	    }
	    crossings[6][1] = val;
	    crossings[6][0] = x2;
	    crossings[6][2] = z2;
	    break;

	case 7:
	    if (DATA[7] == DATA[8])
		val = x1;
	    else {
		val = linterp(DATA[8], DATA[7], threshold, x1);
	    }
	    crossings[7][0] = val;
	    crossings[7][1] = y2;
	    crossings[7][2] = z2;
	    break;

	case 8:
	    if (DATA[8] == DATA[5])
		val = y1;
	    else {
		val = linterp(DATA[5], DATA[8], threshold, y1);
	    }
	    crossings[8][1] = val;
	    crossings[8][0] = x1;
	    crossings[8][2] = z2;
	    break;

	case 9:
	    if (DATA[5] == DATA[1])
		val = z1;
	    else {
		val = linterp(DATA[1], DATA[5], threshold, z1);
	    }
	    crossings[9][2] = val;
	    crossings[9][1] = y1;
	    crossings[9][0] = x1;
	    break;

	case 10:
	    if (DATA[6] == DATA[2])
		val = z1;
	    else {
		val = linterp(DATA[2], DATA[6], threshold, z1);
	    }
	    crossings[10][2] = val;
	    crossings[10][1] = y1;
	    crossings[10][0] = x2;
	    break;

	case 11:
	    if (DATA[8] == DATA[4])
		val = z1;
	    else {
		val = linterp(DATA[4], DATA[8], threshold, z1);
	    }
	    crossings[11][2] = val;
	    crossings[11][1] = y2;
	    crossings[11][0] = x1;
	    break;

	case 12:
	    if (DATA[7] == DATA[3])
		val = z1;
	    else {
		val = linterp(DATA[3], DATA[7], threshold, z1);
	    }
	    crossings[12][2] = val;
	    crossings[12][1] = y2;
	    crossings[12][0] = x2;
	    break;

	}			/* end switch */
	/* next part added by Edward Vigmond */
	if( val< 0 ) {
	    fprintf(stderr, "Error: negative vertex detected (%f), exiting.... \n",val);
	    return (-1);
	}
    }				/* end for */

#define TESTING
	
#ifdef TESTING
    for (i = 0; i < 13; i++) {
	if (crossings[i][0] < 0) {
	    fprintf(stderr, " negative val (%f) at crossing[%d][0], from location %d, %d, %d, index = %d\n",
		    crossings[i][0], i, (int) x1, (int) y1, (int) z1, index);
	    fprintf(stderr, " DATA values: %f,%f,%f,%f,%f,%f,%f,%f \n", 
		DATA[1], DATA[2], DATA[3], DATA[4], DATA[5], DATA[6], DATA[7], DATA[8]);
	    /* crossings[i][0] = 0.; */
	}
	if (crossings[i][1] < 0) {
	    fprintf(stderr, " negative val (%f) at crossings[%d][1], from location %d, %d, %d, index = %d\n",
		    crossings[i][1], i, (int) x1, (int) y1, (int) z1, index);
	    fprintf(stderr, " DATA values: %f,%f,%f,%f,%f,%f,%f,%f \n", 
		DATA[1], DATA[2], DATA[3], DATA[4], DATA[5], DATA[6], DATA[7], DATA[8]);
	    /* crossings[i][1] = 0.; */
	}
	if (crossings[i][2] < 0) {
	    fprintf(stderr, " negative val (%f) at crossings[%d][2], from location %d, %d, %d, index = %d\n",
		    crossings[i][2], i, (int) x1, (int) y1, (int) z1, index);
	    fprintf(stderr, " DATA values: %f,%f,%f,%f,%f,%f,%f,%f \n", 
		DATA[1], DATA[2], DATA[3], DATA[4], DATA[5], DATA[6], DATA[7], DATA[8]);
	    /* crossings[i][2] = 0.; */
	}
    }
#endif

	/* commented out by Ed Vigmond 
    for (i = 0; i < 13; i++)
	if (crossings[i][0] < 0 || crossings[i][1] < 0 || crossings[i][2] < 0) {
	    fprintf(stderr, "Error: negative vertex detected, exiting.... \n");
	    return (-1);
	}
	*/
    return (0);
}

/**************************** get_cell_polys ****************************/

void
get_cell_polys(int index, int *npolys, float crossings[13][3], int x, int y, int z)
/* This subroutine will calculate the polygons */
{
    register int num_o_polys;
    register int poly, idx;
    float    *p1, *p2, *p3;

    num_o_polys = cell_table[index].npolys;	/* possible values: 0 to 10 */

    idx = 0;
    for (poly = 0; poly < num_o_polys; poly++) {

	p1 = &crossings[cell_table[index].polys[idx++]][0];
	p2 = &crossings[cell_table[index].polys[idx++]][0];
	p3 = &crossings[cell_table[index].polys[idx++]][0];

#ifdef DEBUG
	fprintf(stderr, "added polygon from location %d,%d,%d:  %.3f, %.3f, %.3f \n", x, y, z, p1[0], p1[1], p1[2]);
	fprintf(stderr, "added polygon from location %d,%d,%d:  %.3f, %.3f, %.3f \n", x, y, z, p2[0], p2[1], p2[2]);
	fprintf(stderr, "added polygon from location %d,%d,%d:  %.3f, %.3f, %.3f \n", x, y, z, p3[0], p3[1], p3[2]);
#endif

	/* p1,p2,p3 contain x,y,z values for each vertex of the triangle */
	if (add_polygon(p1, p2, p3, x, y, z) == -1) {
	    fprintf(stderr, "%s: error from add_polygon\n", MY_NAME);
	    exit_handler();
	}
    }

    (*npolys) += num_o_polys;

}

/* poly.c              -Brian Tierney, LBL        1/91  */

/* These routines handle the creation of polygons for the isovis program
*
* Portions developed at the National Center for Supercomputing Applications at
* the University of Illinois at Urbana-Champaign.
*
*/

int check_for_duplicate_vertex(float vert[3], int x, int y, int z);
void calc_normal( int x, int y, int z, float v[3], float n[3] );
void get_gradient(int x, int y, int z, float grad[3] );


#define BUF_SIZE 100000
/* allocate space for this many vertices at a time */
#define VERT_INCR 60000		/* size to increase vertex lists when full */

/**************************** add_polygon ****************************/

int
add_polygon(float *p1, float *p2, float *p3, int x, int y, int z)
/* This subroutine stores a polygon (triangle) in a list of vertices */
{

    unsigned  size;
    float     n[3];
    int       idx, conn1, conn2, conn3;
	static int       VERT_LIMIT = BUF_SIZE;
	static int       CONN_LIMIT = BUF_SIZE * 3;

    if (xvertices == NULL) {
	size = VERT_LIMIT * sizeof(float);	/* size for malloc */
	xvertices = alloc_list(size);
	yvertices = alloc_list(size);
	zvertices = alloc_list(size);
    }
    if (DO_NORMALS && xnormals == NULL) {
	size = VERT_LIMIT * sizeof(float);	/* size for malloc */
	xnormals = alloc_list(size);
	ynormals = alloc_list(size);
	znormals = alloc_list(size);
    }
    if (DUP_CHECK && connectivity == NULL) {
	size = CONN_LIMIT * sizeof(int);	/* size for malloc/realloc */
	connectivity = conn_alloc(size, connectivity);
    }
    /* store the vertices */
    if (!DUP_CHECK || (idx = check_for_duplicate_vertex(p1, x, y, z)) < 0) {
	xvertices[num_vertices] = p1[0];	/* x of first vertex */
	yvertices[num_vertices] = p1[1];	/* y of first vertex */
	zvertices[num_vertices] = p1[2];	/* z of first vertex */
	if (DO_NORMALS) {
	    calc_normal(x, y, z, p1, n);
	    xnormals[num_vertices] = n[0];
	    ynormals[num_vertices] = n[1];
	    znormals[num_vertices] = n[2];
	}
	conn1 = num_vertices++;
    } else {
	conn1 = idx;
    }

    if (!DUP_CHECK || (idx = check_for_duplicate_vertex(p2, x, y, z)) < 0) {
	xvertices[num_vertices] = p2[0];	/* x of second vertex */
	yvertices[num_vertices] = p2[1];	/* y of second vertex */
	zvertices[num_vertices] = p2[2];	/* z of second vertex */
	if (DO_NORMALS) {
	    calc_normal(x, y, z, p2, n);
	    xnormals[num_vertices] = n[0];
	    ynormals[num_vertices] = n[1];
	    znormals[num_vertices] = n[2];
	}
	conn2 = num_vertices++;
    } else {
	conn2 = idx;
    }

    if (!DUP_CHECK || (idx = check_for_duplicate_vertex(p3, x, y, z)) < 0) {
	xvertices[num_vertices] = p3[0];	/* x of third vertex */
	yvertices[num_vertices] = p3[1];	/* y of third vertex */
	zvertices[num_vertices] = p3[2];	/* z of third vertex */
	if (DO_NORMALS) {
	    calc_normal(x, y, z, p3, n);
	    xnormals[num_vertices] = n[0];
	    ynormals[num_vertices] = n[1];
	    znormals[num_vertices] = n[2];
	}
	conn3 = num_vertices++;
    } else {
	conn3 = idx;
    }

    if (num_vertices + 3 >= VERT_LIMIT) {
#ifdef MEM_SHORTAGE		/* if have a limitted amount of memory
				 * available, can write out the vertices
				 * gathered so far instead of allocating more
				 * space */

	if (VERBOSE)
	    fprintf(stderr, "\nWriting %d vertices from slice %d \n",
		    num_vertices, z);
	//dump_chunk_polys(1);
#else
	/* get more space */
	VERT_LIMIT += VERT_INCR;/* add this much space */
	size = VERT_LIMIT * sizeof(float);	/* size for malloc/realloc */
	xvertices = realloc_list(size, xvertices);
	yvertices = realloc_list(size, yvertices);
	zvertices = realloc_list(size, zvertices);
	if (DO_NORMALS) {
	    xnormals = realloc_list(size, xnormals);
	    ynormals = realloc_list(size, ynormals);
	    znormals = realloc_list(size, znormals);
	}
#endif
    }
    if (DUP_CHECK && num_conn + 3 >= CONN_LIMIT) {
	CONN_LIMIT += VERT_INCR * 3;	/* add this much space */
	size = CONN_LIMIT * sizeof(int);	/* size for malloc/realloc */
	connectivity = conn_alloc(size, connectivity);
    }
    if (DUP_CHECK) {
	if (currslice[y][x].edge_index == -1)
	    currslice[y][x].edge_index = num_conn;
	currslice[y][x].num_edges += 3;

	/* store the connectivity info */
	connectivity[num_conn++] = conn1;
	connectivity[num_conn++] = conn2;
	connectivity[num_conn++] = conn3;
    }
    return 0;
}

/***********  check for duplicate vertices  *******************************/
int
check_for_duplicate_vertex(float vert[3], int x, int y, int z)
{
    int       idx;


    /* first, check for duplicate vertices from the same cube */
    if ((idx = search_for_dup(vert, currslice[y][x].num_edges,
			      currslice[y][x].edge_index)) >= 0)
	return (idx);

    if (x > 0) {
	if ((idx = search_for_dup(vert, currslice[y][x - 1].num_edges,
				  currslice[y][x - 1].edge_index)) >= 0)
	    return (idx);
    }
    if (y > 0) {
	if ((idx = search_for_dup(vert, currslice[y - 1][x].num_edges,
				  currslice[y - 1][x].edge_index)) >= 0)
	    return (idx);
    }
    if (z > 0) {
	if ((idx = search_for_dup(vert, prevslice[y][x].num_edges,
				  prevslice[y][x].edge_index)) >= 0)
	    return (idx);
    }
    return (-1);
}

 /***************************************************************/
int
search_for_dup(float vert[3], int num_to_check, int conn_idx)
{
    register int i, idx;
    static int dup_cnt = 0;

    if (conn_idx == -1)
	return (-1);

    if (num_to_check == -1) {	/* for testing */
	fprintf(stderr, " found %d duplicate vertices \n", dup_cnt);
	return (-1);
    }
    for (i = 0; i < num_to_check; i++) {
	idx = connectivity[conn_idx];
	if (idx > num_vertices) {	/* seems to need this check, but I
					 * dont know why? */
	    fprintf(stderr, "Error checking dup vertex: conn_idx = %d \n",
		    conn_idx);
	    return (-1);
	}
	if (DIFF > 0.0) {
	    if (fabs(xvertices[idx] - vert[0]) < DIFF &&
		fabs(yvertices[idx] - vert[1]) < DIFF &&
		fabs(zvertices[idx] - vert[2]) < DIFF) {
		dup_cnt++;
		return (idx);
	    }
	} else if (xvertices[idx] == vert[0] &&
		   yvertices[idx] == vert[1] &&
		   zvertices[idx] == vert[2]) {
	    dup_cnt++;
	    return (idx);
	}
	conn_idx++;
    }

    return (-1);		/* none found */
}

/**************************** calc_normal ****************************/

void
calc_normal(int x, int y, int z, float v[3], float n[3])
{
    float     sum, mag;

#define METHOD2

#ifdef METHOD1
    float     g1[3], g2[3];
    /*
     * Method 1: this method uses the average of the gradients of the 2
     * points closest to the computed vertex
     */
    get_gradient(x, y, z, g1);
    if (v[0] - x - 1 < 0) {	/* subtract 1 because verts start at 1 */
	get_gradient(x + 1, y, z, g2);
    } else if (v[1] - y - 1 < 0) {
	get_gradient(x, y + 1, z, g2);
    } else if (v[2] - z - 1 < 0) {
	get_gradient(x, y, z + 1, g2);
    } else {
	fprintf(stderr, " ???????: xyz=%d,%d,%d, vert: %f,%f,%f\n",
		x, y, z, v[0], v[1], v[2]);
	g2[0] = g2[1] = g2[2] = 0.;
    }

    n[0] = g2[0] + g1[0] / 2.;
    n[1] = g2[1] + g1[1] / 2.;
    n[2] = g2[2] + g1[2] / 2.;
#endif

#ifdef METHOD2
    /*
     * this method just uses the gradient of the closest vertex to the
     * triagle vertex. This method is faster and gives better results (I
     * think) than method 1
     */
    get_gradient((int) (v[0] + .5), (int) (v[1] + .5),
		 (int) (v[2] + .5), n);

#endif

    if (DO_NORMALS == 2) {	/* 2 means to do 'normalized normals', some
				 * rendering packages may require this (SCRY
				 * does not */
	sum = n[0] * n[0] + n[1] * n[1] + n[2] * n[2];
	mag = (float) sqrt((double) sum);

	if (mag == 0.0)
	    mag = 1.0;

	n[0] = n[0] / mag;
	n[1] = n[1] / mag;
	n[2] = n[2] / mag;
    }
#ifdef DEBUG
    fprintf(stderr, " normal values are: %f, %f, %f \n", n[0], n[1], n[2]);
#endif
}

/* poly.c              -Brian Tierney, LBL        1/91  */

/* These routines handle the creation of polygons for the isovis program
*
* Portions developed at the National Center for Supercomputing Applications at
* the University of Illinois at Urbana-Champaign.
*
*/

int check_for_duplicate_vertex(float vert[3], int x, int y, int z);
void calc_normal( int x, int y, int z, float v[3], float n[3] );
void get_gradient(int x, int y, int z, float grad[3] );


/******************************************************************/
void
get_gradient(int x, int y, int z, float grad[3])
/* This routine computes the gradient at location x,y,z, and returns
 *    this value in grad.
 */
{
    /* compute gradients using center differences */

    /* NOTE: this is not the fastest or most efficient way to do this !! */
    /*
     * a better method might be to compute all gradients while doing the
     * segmentation
     */

    if (x == 0)			/* use edge value instead */
	grad[0] = (data[z][y][x + 1] - data[z][y][x]) / 2.;
    else if (x >= xmax)
	grad[0] = (data[z][y][x] - data[z][y][x - 1]) / 2.;
    else
	grad[0] = (data[z][y][x + 1] - data[z][y][x - 1]) / 2.;

    if (y == 0)
	grad[1] = (data[z][y + 1][x] - data[z][y][x]) / 2.;
    else if (y >= ymax)
	grad[1] = (data[z][y][x] - data[z][y - 1][x]) / 2.;
    else
	grad[1] = (data[z][y + 1][x] - data[z][y - 1][x]) / 2.;

    if (z == 0)
	grad[2] = (data[z + 1][y][x] - data[z][y][x]) / 2.;
    else if (z >= zmax)
	grad[2] = (data[z][y][x] - data[z - 1][y][x]) / 2.;
    else
	grad[2] = (data[z + 1][y][x] - data[z - 1][y][x]) / 2.;
}

/**************************** dump_polys ****************************/
void
Isosurf::dump_chunk_polys(FILE *out, int more, int output_type)
{
	poly_fp = out;
    if (output_type == 1) {
	write_polys_ascii(xvertices, yvertices, zvertices,
			  xnormals, ynormals, znormals,
			  num_vertices, connectivity, num_conn);
    } else if (output_type == 2) {
	write_polys_scry(xvertices, yvertices, zvertices,
			 xnormals, ynormals, znormals,
			 num_vertices, connectivity, num_conn);
    } else if (output_type == 0) {
	write_poly_header();
	write_polys_xgl(xvertices, yvertices, zvertices,
			xnormals, ynormals, znormals,
			num_vertices, more);
    }
    num_vertices = num_conn = 0;
}

/* segment.c                Brian Tierney, LBL
 */

#include <memory.h>

int       stack_size;
STACK_ITEM *stack = NULL;
int       sp = 0;		/* stack pointer */

void clear_background_grid(Grid_type ** grid2d, int sx, int sy, int ex, int ey);
void reset_grid(Grid_type ***grid3d, int sx, int sy, int ex, int ey, int chunk_size);
int get_seed_guess1(Grid_type *** grid3d, int *sx, int *sy, int *sz, int startx, 
		int starty, int endx, int endy, int chunk_size);
int get_seed_guess2(Grid_type ***grid3d, int *sx, int *sy, int *sz, int startx, 
			int starty, int endx, int endy, int chunk_size);
int get_second_chunk_seed(Grid_type **grid2d, int *sx, int *sy, int startx, 
						int starty, int endx, int endy);
int check_for_neighbors(Grid_type **grid2d, int c, int r, int endx, int endy);
int object_fill(Grid_type ***grid3d, int c, int r, int f, int endx, int endy, 
				int chunk_size);
int count_neighbor_edges(Grid_type ***grid3d, int c, int r, int f, int ncol, 
							int nrow, int chunk_size);
void locate_surfaces(Grid_type **grid2d, int sx, int sy, int ex, int ey);
STACK_ITEM * alloc_stack(int st_size);	// allocation of stack for non-recursive
void show_grid_slice(Grid_type **grid2d, int sx, int sy, int ex, int ey);

#define MIN_AREA .01		/* minimum percentage of total area to keep
				 * an object */


/* a 'grid' is used to perform the segmentation. The meaning of the
 *  grid values are as follows:
 *     1 : thresholded object
 *     2 : edge of object  (if DO_BRIDGES is on )
 *     3 : flood fill object
 *     4 : final object
 */

/* #define SEG_DEBUG */

/***************************************************************/
int
seg_slice_thresh(float thresh, float **data2d, Grid_type **grid2d, 
		int sx, int sy, int ex, int ey, int slice)
{
    /* simple thresholding to segment data */

    register int i, j, k;
    register int zero_cnt = 0, skip;
    int       stop;
    Grid_type *save_ptr = NULL;
    int       num_on = 0;


#ifdef SEG_DEBUG
    fprintf(stderr, "segmenting slice %d \n", slice);
#endif

    if (SKIP_HOLES) {
	for (j = sy; j < ey; j++) {
	    if (zero_cnt > 0) {
		skip = -(zero_cnt - 1);
		stop = zero_cnt - 2;
		for (k = 0; k < stop; k++)
		    *save_ptr++ = skip++;
		zero_cnt = 0;
	    }
	    for (i = sx; i < ex; i++) {
		if (data2d[j][i] >= thresh) {
		    num_on++;
		    grid2d[j][i] = 1;
		    if (zero_cnt > 0) {
			skip = -(zero_cnt - 1);
			stop = zero_cnt - 2;
			/*
			 * store # of cubes to skip in grid2d cell as a
			 * negative number
			 */
			for (k = 0; k < stop; k++)
			    *save_ptr++ = skip++;
			zero_cnt = 0;
		    }
		} else {
		    grid2d[j][i] = 0;
		    zero_cnt++;
		    if (zero_cnt == 1)
			save_ptr = &(grid2d[j][i]);
		}
	    }
	}
    } else {
	for (j = sy; j < ey; j++)
	    for (i = sx; i < ex; i++) {
		if (data2d[j][i] >= thresh) {
		    grid2d[j][i] = 1;
		    num_on++;
		} else {
		    grid2d[j][i] = 0;
		}
	    }
    }

    if (VERBOSE)
	fprintf(stderr, "%d voxels selected from slice %d \n", num_on, slice);

#ifdef SEG_DEBUG
    fprintf(stderr, "after thresholding... \n");
    show_grid_slice(grid2d, sx, sy, ex, ey);
#endif

    return (num_on);
}

/*************************************************************/
int
flood_fill_segmentation(int startx, int starty, int endx, int endy, 
													int slice, int chunk_size)
{
    int       rval1, rval2, sx, sy, sz, pcnt, min_size;
    register int i;
    static int slice_cnt = 0;

    slice_cnt++;
    pcnt = 0;
    sx = sy = sz = rval1 = rval2 = 0;
    min_size = int(endx * endy * MIN_AREA);	/* at least 5% of pixels */

    stack_size = int((endx - startx) * (endy - starty) * chunk_size * .8);
    stack = alloc_stack(stack_size);

    if (DO_BRIDGES) {
	for (i = 0; i <= chunk_size; i++) {
	    /* label edges of object '2' */
	    locate_surfaces(grid[i], startx, starty, endx, endy);

#ifdef SEG_DEBUG
	    fprintf(stderr, "after edge labelling... \n");
	    show_grid_slice(grid[i], startx, starty, endx, endy);
#endif
	}
    }
    while (pcnt < min_size) {
	if (rval1 == 0)
	    rval1 = get_seed_guess1(grid, &sx, &sy, &sz, startx, starty,
				    endx, endy, chunk_size);

	if (rval1 < 0)
	    rval2 = get_seed_guess2(grid, &sx, &sy, &sz, startx, starty,
				    endx, endy, chunk_size);

	if (rval2 < 0) {	/* didn't find an object */
	    memset((char *) grid[0][0], 0, endx * endy * sizeof(Grid_type));
	    return (-1);	/* give up */
	}
	pcnt = object_fill(grid, sx, sy, sz, endx, endy, chunk_size);
	/* grid = -1 if desired object */

	if (pcnt < min_size)	/* object not big enough, reset and try again */
	    reset_grid(grid, startx, starty, endx, endy, chunk_size);

	if (VERBOSE && pcnt >= min_size) {
	    fprintf(stderr, "Slice %d: Object at location: (%d,%d);  %d pixels \n",
		    slice, sx, sy, pcnt);
	}
    }
    for (i = 0; i <= chunk_size; i++) {

#ifdef SEG_DEBUG
	fprintf(stderr, "after fill... \n");
	show_grid_slice(grid[i], startx, starty, endx, endy);
#endif

	/* grid2d = 0 if grid2d != 3 */
	clear_background_grid(grid[i], startx, starty, endx, endy);

#ifdef SEG_DEBUG
	fprintf(stderr, "final segmentation \n");
	show_grid_slice(grid[i], startx, starty, endx, endy);
#endif
    }

    free((char *) stack);
    return (pcnt);

}

/*********************************************************/
/*  routines for identifing objects using the grid       */
/*********************************************************/

void
clear_background_grid(Grid_type **grid2d, int sx, int sy, int ex, int ey)
 /*
  * at this point, all points in the desired object should be 3, so set any
  * other points to 0
  */
{
    register int i, j;

    for (j = sy; j < ey; j++)
	for (i = sx; i < ex; i++) {
	    if (grid2d[j][i] > 0) {
		if (DO_BRIDGES) {
		    if (grid2d[j][i] == 3 ||
			(grid2d[j][i] == 2 &&
			 check_for_neighbors(grid2d, i, j, ex, ey) == 1))
			grid2d[j][i] = 4;
		    else
			grid2d[j][i] = 0;
		} else {
		    if (grid2d[j][i] == 3)
			grid2d[j][i] = 4;
		    else
			grid2d[j][i] = 0;
		}
	    }
	}
}

/****************************************************************/
void
reset_grid(Grid_type ***grid3d, int sx, int sy, int ex, int ey, int chunk_size)
 /*
  * resets objects marked '3' to '0' if determined that the object was too
  * small to use
  */
{
    register int i, j, k;

    for (k = 0; k < chunk_size; k++)
	for (j = sy; j < ey; j++)
	    for (i = sx; i < ex; i++)
		if (grid3d[k][j][i] == 3)
		    grid3d[k][j][i] = 0;
}

/***************************************************************/

int
get_seed_guess1(Grid_type ***grid3d, int *sx, int *sy, int *sz, int startx, 
		int starty, int endx, int endy, int chunk_size)
{
/* looks for a good seed value starting near the center of the image */
    register int f, r, c;

    int       bx, by, bz, endz;

    bx = (endx - startx) / 2;	/* start at center of data set */
    by = (endy - starty) / 2;
    bz = chunk_size / 2;

#ifdef DEBUG
    fprintf(stderr, "\nguess1: Looking for a seed value starting at: %d,%d \n",
	    bx, by);
#endif

    endy = endy - 1;
    endx = endx - 1;
    endz = chunk_size - 1;

    for (f = bz; f < endz; f++)
	for (r = by; r < endy; r++)
	    for (c = bx; c < endx; c++)
		if (grid3d[f][r][c] == 1) {
		    if ((grid3d[f][r][c - 1] == 1) &&
			(grid3d[f][r][c + 1] == 1) &&
			(grid3d[f][r - 1][c] == 1) &&
			(grid3d[f][r + 1][c] == 1)) {
			*sx = bx = c;
			*sy = by = r;
			*sz = bz = f;
			return (0);
		    }
		}
    return (-1);
}

/***************************************************************/

int
get_seed_guess2(Grid_type ***grid3d, int *sx, int *sy, int *sz, int startx, 
			int starty, int endx, int endy, int chunk_size)
{
    /*
     * looks for a good seed value starting at the upper left corner of the
     * image
     */

    register int f, r, c;
    int       bx = startx + 1, by = starty + 1, bz = 1, endz;

#ifdef DEBUG
    fprintf(stderr, "\nguess2: Looking for a seed value starting at: %d,%d \n",
	    bx, by);
#endif

    endy = endy - 1;
    endx = endx - 1;
    endz = chunk_size - 1;

    for (f = bz; f < endz; f++)
	for (r = by; r < endy; r++)
	    for (c = bx; c < endx; c++)
		if (grid3d[f][r][c] == 1) {
		    if ((grid3d[f][r][c - 1] == 1) &&
			(grid3d[f][r - 1][c] == 1) &&
			(grid3d[f][r][c + 1] == 1) &&
			(grid3d[f][r + 1][c] == 1)) {
			*sx = bx = c;
			*sy = by = r;
			*sz = bz = f;
			return (0);
		    }
		}
    return (-1);
}

/***************************************************************/

int
get_second_chunk_seed(Grid_type **grid2d, int *sx, int *sy, int startx, 
						int starty, int endx, int endy)
{
    register int r, c;

    endy = endy - 1;
    endx = endx - 1;

    for (r = starty; r < endy; r++)
	for (c = startx; c < endx; c++)
	    if (grid2d[r][c] == 3) {
		*sx = c;
		*sy = r;
		return (0);
	    }
    return (-1);
}

/***************************************************************/
int
check_for_neighbors(Grid_type **grid2d, int c, int r, int endx, int endy)
{
    /*
     * returns a 1 if any neighbor is a 3 ( selected by fill )
     */

    int       tx, ty;

    tx = endx - 1;
    ty = endy - 1;

    if (r > 0)
	if (grid2d[r - 1][c] >= 3)
	    return (1);

    if (r < ty)
	if (grid2d[r + 1][c] >= 3)
	    return (1);

    if (c > 0)
	if (grid2d[r][c - 1] >= 3)
	    return (1);

    if (c < tx)
	if (grid2d[r][c + 1] >= 3)
	    return (1);

#ifdef DIAG
    /* diagonal neighbors */
    if (r > 0 && c > 0)
	if (grid2d[r - 1][c - 1] >= 3)
	    return (1);

    if (r > 0 && c < tx)
	if (grid2d[r - 1][c + 1] >= 3)
	    return (1);

    if (r < ty && c > 0)
	if (grid2d[r + 1][c - 1] >= 3)
	    return (1);

    if (r < ty && c < tx)
	if (grid2d[r + 1][c + 1] >= 3)
	    return (1);
#endif

    return (0);
}

/***************************************************************/
/* routines for handling stack-based recursion                 */
/***************************************************************/
int
object_fill(Grid_type ***grid3d, int c, int r, int f, int endx, int endy, 
				int chunk_size)
{
    /* fill the objects with (3) */
    int       ff, rr, cc, pt_count = 0;
    int       endz = chunk_size;

    sp = 0;			/* initialize stack pointer */
    push(-1, -1, -1);		/* null stack */


    do {
start:
	if (DO_BRIDGES && pt_count > 1) {	/* check strength of
						 * connecting bridges */
	    if (grid3d[f][r][c] == 2) {
		if (count_neighbor_edges(grid, c, r, f, endx, endy, chunk_size) > 2) {
		    goto w_stop;
		}
	    }
	}
	grid3d[f][r][c] = 3;	/* mark grid */
	pt_count++;

	ff = f + 1;
	if ((f < endz - 1) && (grid3d[ff][r][c] == 1)) {
	    push(c, r, f);
	    f++;
	    goto start;
	}
	ff = f - 1;
	if ((ff >= 0) && (grid3d[ff][r][c] == 1)) {
	    push(c, r, f);
	    f--;
	    goto start;
	}
	rr = r + 1;
	if (rr < endy && grid3d[f][rr][c] == 1) {
	    push(c, r, f);
	    r++;
	    goto start;
	}
	rr = r - 1;
	if (rr >= 0 && grid3d[f][rr][c] == 1) {
	    push(c, r, f);
	    r--;
	    goto start;
	}
	cc = c + 1;
	if (cc < endx && grid3d[f][r][cc] == 1) {
	    push(c, r, f);
	    c++;
	    goto start;
	}
	cc = c - 1;
	if (cc >= 0 && grid3d[f][r][cc] == 1) {
	    push(c, r, f);
	    c--;
	    goto start;
	}
w_stop:
	pop(&c, &r, &f);

    } while (r >= 0);		/* neg r indicates empty stack */

    if (sp != 0)
	fprintf(stderr, "Error: stack not empty \n");

    return (pt_count);
}

/***************************************************************/
void
locate_surfaces(Grid_type **grid2d, int sx, int sy, int ex, int ey)
{				/* mark all surfaces as 2 */
    register int c, r, ex2, ey2;

    ex2 = ex - 1;
    ey2 = ey - 1;

    /* label the surface points as 2  */

    for (r = sy; r < ey; r++)
	for (c = sx; c < ex; c++)
	    if (grid2d[r][c] == 1) {
		if ((r > 0 && grid2d[r - 1][c] == 0) ||
		    (r < ey2 && grid2d[r + 1][c] == 0) ||
		    (c > 0 && grid2d[r][c - 1] == 0) ||
		    (c < ex2 && grid2d[r][c + 1] == 0) ||
		    (r == 0 || c == 0 || r == ey2 || c == ex2))
		    grid2d[r][c] = 2;
	    }
}

/***************************************************************/
int
count_neighbor_edges(Grid_type ***grid3d, int c, int r, int f, int ncol, 
							int nrow, int chunk_size)
{
    int       neighbors = 0, tx, ty, tz;

    tx = ncol - 1;
    ty = nrow - 1;
    tz = chunk_size - 1;

    if (r > 0)
	if (grid3d[f][r - 1][c] == 2 || grid3d[f][r - 1][c] == 0)
	    neighbors++;
    if (c > 0)
	if (grid3d[f][r][c - 1] == 2 || grid3d[f][r][c - 1] == 0)
	    neighbors++;

    if (r < ty)
	if (grid3d[f][r + 1][c] == 2 || grid3d[f][r + 1][c] == 0)
	    neighbors++;
    if (c < tx)
	if (grid3d[f][r][c + 1] == 2 || grid3d[f][r][c + 1] == 0)
	    neighbors++;

    /* previous slice */
    if ((f > 0) && (r > 0))
	if (grid3d[f - 1][r - 1][c] == 2)
	    neighbors++;
    if ((f > 0) && (c > 0))
	if (grid3d[f - 1][r][c - 1] == 2)
	    neighbors++;
    if ((f > 0) && (r < ty))
	if (grid3d[f - 1][r + 1][c] == 2)
	    neighbors++;
    if ((f > 0) && (c < tx))
	if (grid3d[f - 1][r][c + 1] == 2)
	    neighbors++;

    /* next slice */
    if ((f < tz) && (r > 0))
	if ((grid3d[f + 1][r - 1][c] == 2))
	    neighbors++;
    if ((f < tz) && (c > 0))
	if ((grid3d[f + 1][r][c - 1] == 2))
	    neighbors++;
    if ((f < tz) && (r < ty))
	if ((grid3d[f + 1][r + 1][c] == 2))
	    neighbors++;
    if ((f < tz) && (c < tx))
	if ((grid3d[f + 1][r][c + 1] == 2))
	    neighbors++;

    return (neighbors);
}

/***************************************************************/
void
push(int i, int j, int k)
{
    sp++;
    if (sp >= stack_size) {
	fprintf(stderr, "recursive stack overflow!! ");
	fprintf(stderr, " stack was allocated %d slots \n", stack_size);
	exit_handler();
    }
    stack[sp].i = i;
    stack[sp].j = j;
    stack[sp].k = k;
}

/***************************************************************/
void
pop(int *i, int *j, int *k)
{
    *i = stack[sp].i;
    *j = stack[sp].j;
    *k = stack[sp].k;
    sp--;
}

/***************************************************************/
STACK_ITEM *
alloc_stack(int st_size)		/* allocation of stack for non-recursive
				 * flood-fill alg */
{
    if ((stack = Calloc(st_size, STACK_ITEM)) == NULL)
	perror("calloc: stack");

    return (stack);
}

/***************************************************************/
void show_grid_slice(Grid_type **grid2d, int sx, int sy, int ex, int ey)
{				/* for debugging  */
    register int c, r;

    fprintf(stderr, "\n");
    for (r = sy; r < ey; r++) {
	for (c = sx; c < ex; c++)
	    fprintf(stderr, "%2d", grid2d[r][c]);
	fprintf(stderr, "\n");
    }
}

/* main.c  */

/* This program implements the marching cubes surface tiler described by
 * Lorensen & Cline in the Siggraph 87 Conference Proceedings.
 *
 * Original program written by Mike Krogh, NCSA, Feb.  2, 1990
 * Portions developed at the National Center for Supercomputing Applications at
 * the University of Illinois at Urbana-Champaign.
 *
 * modified by Brian Tierney, LBL  12/90
 *            Lawrence Berkeley Laboratory
 *            Imaging Technologies Group
 *            email: bltierney@lbl.gov
 *  changes include removal of SGI Iris specific code, added code to
 *  remove duplicate vertices and compute normals, and various
 *  speed enhancements.
 *
 *    see usage routine below for command line options
 *
 */

void write_mask_slices(int num_slices);

/*****************************************************/
void
tessellate_volume( float l_tval )
{
    Grid_type **tmp_ptr;
    int       chunk_cnt, num_chunks;
    int       num_voxels[CHUNK_SIZE], npolys = 0;
    int       size, start_slice, rval;
    int       num_slices_per_chunk, slice;
    float     elapsed_time;
    register int i;

#ifdef CRAY
#define DHZ 166722200.0		/* PSC Cray Y-MP clock time = 5.998 ns; DHZ =
				 * 1/clock */
    struct tms tbuf;
    long      start_time, end_time;
#else
    struct timeb start_time, end_time;
#endif
	if (SX > 0 || SY > 0 || SZ > 0 || EX < xdim || EY < ydim || EZ < zdim) {
	SMALLER_BOX = 1;	/* global flag indication using reduced
				 * volume */
	fprintf(stderr, "Using partial input data set: from %d,%d,%d to %d,%d,%d\n",
		SX, SY, SZ, EX, EY, EZ);
    }
    if (EX > xdim)		/* set-up for handling partial volumes */
	EX = xdim;
    if (EY > ydim)
	EY = ydim;
    if (EZ > zdim)
	EZ = zdim;

    /* setup globals */
    xmax = xdim - 1;
    ymax = ydim - 1;
    zmax = zdim - 1;

    size = ydim * xdim;

    if (CHUNK_SIZE < (EZ - SZ)) {
	num_slices_per_chunk = CHUNK_SIZE - 1;
	if (xdim % num_slices_per_chunk == 0)
	    num_chunks = (EZ - SZ) / num_slices_per_chunk;
	else
	    num_chunks = ((EZ - SZ) / num_slices_per_chunk) + 1;
    } else {
	num_slices_per_chunk = EZ - SZ - 1;
	num_chunks = 1;
    }
#ifdef DEBUG
    fprintf(stderr, " num_chunks: %d, slices per chunk: %d \n", num_chunks,
	    num_slices_per_chunk);
#endif

    if (DUP_CHECK && prevslice == NULL) {	/* for checking for dup verts */
	prevslice = alloc_2d_cube_array(xdim, ydim);
	currslice = alloc_2d_cube_array(xdim, ydim);
    }
#ifdef SPEED
#ifdef CRAY
    start_time = times(&tbuf);
#else
    ftime(&start_time);
#endif
#endif

	if (VERBOSE) 
    	fprintf(stderr, "%s: looking for surfaces at iso-value: %.3f \n",
			MY_NAME, l_tval);

    slice = start_slice = SZ;

    for (chunk_cnt = 0; chunk_cnt < num_chunks; chunk_cnt++) {

	for (i = 0; i <= num_slices_per_chunk; i++) {
	    if (VERBOSE)
		fprintf(stderr, "thresholding slice: %d \n", slice);

	    num_voxels[i] = seg_slice_thresh(l_tval, Isosurf::data[slice], grid[i],
					     SX, SY, EX, EY, slice);
	    slice++;
	}

	slice = start_slice;
	if (SEG_METHOD == 1) {	/* segment entire chunk */
	    if (num_slices_per_chunk < zmax) {
		fprintf(stderr, "Flood segmentation with small chunk size not yet implemented \n");
		fprintf(stderr, "Using thresholding only...... \n");
	    } else {
		flood_fill_segmentation(SX, SY, EX, EY,
					slice, num_slices_per_chunk);
	    }
	}
	for (i = 0; i < num_slices_per_chunk; i++) {
	    /* dont run on last slice per the chunk */
	    if (VERBOSE)
		fprintf(stderr, "tessellating slice: %d \n", slice);

	    if (num_voxels[i] > 0) {
		if ((rval = iso_surface(SX, SY, EX, EY, i,
					slice, l_tval)) < 0) {
		    /* Error detected, but write what we got so far */
		    ////dump_chunk_polys(0);
		    return;
		} else
		    npolys += rval;
	    }
	    slice++;
	}
	start_slice = slice;

	if (slice + num_slices_per_chunk > (EZ - 1))
	    num_slices_per_chunk = EZ - slice - 1;

	/* shift pointer to move slices down one */
#ifdef MOVE_PTR_METHOD
	/*
	 * using this method will only work in non-server version or if we
	 * add some code to restore the pointers to their original state.
	 * Otherwise the program will die on calls to 'free' for the grid in
	 * the server version
	 */
	tmp_ptr = grid[0];
	grid[0] = grid[CHUNK_SIZE - 1];
	grid[CHUNK_SIZE - 1] = tmp_ptr;
#else
	memcpy((char *) grid[0][0], (char *) grid[CHUNK_SIZE - 1][0],
	       xdim * ydim * sizeof(Grid_type));
#endif
   }				/* for each chunk */
#ifdef SPEED
#ifdef CRAY
    end_time = times(&tbuf);
    fprintf(stderr, "\nTime for tessellation: %f seconds\n", (end_time - start_time) / DHZ);
#else
    ftime(&end_time);
    elapsed_time = ((float) ((end_time.time * 1000) + end_time.millitm) -
		    ((start_time.time * 1000) + start_time.millitm)) / 1000.;
    fprintf(stderr, "\nTime for tessellation: %f seconds\n", elapsed_time);
#endif
#endif

    //dump_chunk_polys(0);	/* make sure all vertices have been written */

	if( VERBOSE )
    	fprintf(stderr, "\n%s: %d polygons generated\n", MY_NAME, npolys);

    return;


}


/************************************************************************/
void 
write_poly_header()
{
    int       n = 1;

    Fwrite(&n, sizeof(int), 1, poly_fp);
    /* write size of input volume */
    Fwrite(&xdim, sizeof(int), 1, poly_fp);
    Fwrite(&ydim, sizeof(int), 1, poly_fp);
    Fwrite(&zdim, sizeof(int), 1, poly_fp);
    if (DO_NORMALS)
	n = 1;
    else
	n = 0;
    Fwrite(&n, sizeof(int), 1, poly_fp);

}

/* misc.c */
/*
* Portions developed at the National Center for Supercomputing Applications at
* the University of Illinois at Urbana-Champaign.
*/

#include <malloc.h>

/********************************************************/
float    *
alloc_list(unsigned int size)
/* This subroutine is for allocating memory for the vertex lists */
{
    float    *buf;

    if ((buf = (float *) malloc(size)) == NULL) {
	fprintf(stderr, "%s: error, not enough memory to store vertices\n",
		MY_NAME);
	exit_handler();
    }
    return (buf);
}

/********************************************************/
float    *
realloc_list(unsigned int size, float *buf)
{
    if ((buf = (float *) realloc((char *) buf, size)) == NULL) {
	fprintf(stderr, "%s: error, not enough memory to store vertices\n",
		MY_NAME);
	exit_handler();
    }
    return (buf);
}

/**************************** conn_alloc ****************************/

int      *
conn_alloc(unsigned int size, int *buf)
/* This subroutine is for allocating memory for the connectivity list */
{

    /* flag to allocate memory from 'malloc' first time through */
    static int first_alloc = 1;

    if (first_alloc) {
	/* use 'malloc' for the first time thru */
	if ((buf = (int *) malloc(size)) == NULL) {
	    fprintf(stderr, "%s: error, not enough memory to store connectivity\n",
		    MY_NAME);
	    exit_handler();
	}
	first_alloc = 0;	/* turn off this flag */
    } else {
	/* use 'realloc' from now on */
	if ((buf = (int *) realloc((char *) buf, size)) == NULL) {
	    fprintf(stderr, "%s: error, not enough memory to store connectivity\n",
		    MY_NAME);
	    exit_handler();
	}
    }

    return (buf);
}


/*********************************************************/
CUBE_TRIANGLES**
alloc_2d_cube_array(int nx, int ny)
{
    CUBE_TRIANGLES **array;
    register int i;

    /* allocate 2-d array for input image data */
    /* allocate array of pointers */
    if ((array = Calloc(nx, CUBE_TRIANGLES *)) == NULL)
	perror("calloc error: cube array ");

    /* allocate array for data */
    if ((array[0] = Calloc(nx * ny, CUBE_TRIANGLES)) == NULL)
	perror("calloc error: cube array ");

    /* initialize pointer arrays */
    for (i = 1; i < nx; i++)
	array[i] = array[0] + (ny * i);

    return (array);
}

/**************************************************/

/***********************************************************/
void
alloc_3d_grid_array(int nx, int ny, int nz)	
{
    register int i, j;

    /* allocate 3-d array for input image data */

    /* allocate 2 arrays of pointers */
    if ((grid = Calloc(nx, Grid_type **)) == NULL)
	perror("calloc error: grid array ");
    if ((grid[0] = Calloc(nx * ny, Grid_type *)) == NULL)
	perror("calloc error: grid array ");

    /* allocate array for data */
    if ((grid[0][0] = Calloc(nx * ny * nz, Grid_type)) == NULL)
	perror("calloc error: grid array ");

    /* initialize pointer arrays */
    for (i = 1; i < ny; i++)
	grid[0][i] = **grid + (nz * i);
    for (i = 1; i < nx; i++) {
	grid[i] = *grid + (ny * i);
	grid[i][0] = **grid + (nz * ny * i);
	for (j = 1; j < ny; j++)/* initialize pointer array */
	    grid[i][j] = grid[i][0] + (nz * j);
    }

}

/**********************************/
float   **
alloc_2d_float_array(int nx, int ny)
{
    float   **array;
    register int i;

    /* allocate 2-d array for input image data */
    /* allocate array of pointers */
    if ((array = Calloc(nx, float *)) == NULL)
	perror("calloc error: array ");

    /* allocate array for data */
    if ((array[0] = Calloc(nx * ny, float)) == NULL)
	perror("calloc error: array ");

    /* initialize pointer arrays */
    for (i = 0; i < nx; i++)
	array[i] = *array + (ny * i);

    return (array);
}

/********************************/
void
free_3d_grid_array()
{
#ifdef CRAY
    cfree((char *) grid[0][0], NULL, NULL);
    cfree((char *) grid[0], NULL, NULL);
    cfree((char *) grid, NULL, NULL);
#else
    free((char *) grid[0][0]); 
    free((char *) grid[0]);
    free((char *) grid);
#endif
}

/*********************************/
int
free_2d_float_array(float   **array)
{
#ifdef CRAY
    cfree((char *) array[0], NULL, NULL);
    cfree((char *) array, NULL, NULL);
#else
    free((char *) array[0]);
    free((char *) array);
#endif
}
